/*======================================================================
 * bootstrap - storage and retrieval for ancillary information needed
 *               to run the Bootstrap sea ice algorithms.
 *
 * 02/11/2000 brodzik brodzik@zamboni.colorado.edu 303-492-8263
 * National Snow & Ice Data Center, University of Colorado, Boulder
 *
 *$Revision: 1.1.8.1 $
 *
 *======================================================================*/
#ifndef bootstrap_h_
#define bootstrap_h_

#include "define.h"
#include "pmalgorithm.h"
#include "land_spillover.h"

#define BOOTSTRAP_SAT_NAME_LENGTH 10
#define BOOTSTRAP_DATE_LENGTH 9
#define BOOTSTRAP_VERSION_LENGTH 11
#define BOOTSTRAP_VERSION_01 "ver1"
#define BOOTSTRAP_VERSION_02 "ver2"

static const char bootstrap_h_rcsid[] = "$Header: /tmp_mnt/FILES/pmalgos/bootstrap.h,v 1.1.8.1 2008-06-10 20:36:35 brodzik Exp $";

typedef enum {
    BOOTSTRAP_NORTH_HEM = 1, BOOTSTRAP_SOUTH_HEM
} bootstrap_hemisphere;

typedef enum {
    BOOTSTRAP_DEFAULT = 0,
    BOOTSTRAP_F08, BOOTSTRAP_F11, BOOTSTRAP_F13, BOOTSTRAP_F17_FINAL, BOOTSTRAP_NUM_SATS
} bootstrap_sat;

static const char *bootstrap_sat_name[] = {"DEFAULT",
                                           "F08",
                                           "F11",
                                           "F13",
                                           "F17_final"};


/* Structure for ancillary algorithm information */
typedef struct {
  bootstrap_hemisphere hem;                /* Hemisphere id */
  char sat_name[BOOTSTRAP_SAT_NAME_LENGTH];/* Satellite id */
  char date[BOOTSTRAP_DATE_LENGTH];        /* Date of input tbs, yyyymmdd */
  grid_class *grid;                        /* grid parameters */
  char algo_version[BOOTSTRAP_VERSION_LENGTH];  /* algorithn version */
  byte1 **landmask;                        /* landmask array */
  byte2 **seaice;                          /* seaice array */
  bool scale_percent_seaice;               /* flag for scaling seaice values to 0-100% */
  land_spillover_class *land_spillover;    /* land spillover class */
  int bootstrap_recoding_pcent;            /* minimum seaice threshold for bootstrap recoding */
  int verbose;                             /* verbosity flag */
} bootstrap_class;
  
/* Function prototypes */
bool is_bootstrap_algorithm(pmalgo_id algorithm);
bool is_bootstrap_v02_algorithm(pmalgo_id algorithm);
bool grid_is_polarstero(grid_class* grid);
bool valid_v2_environment(void);
bootstrap_class *init_bootstrap(const char *algorithm_name,
                                const char *sat_name,
                                const char *date, 
                                const int bootstrap_recoding_pcent,
                                const bool scale_percent_seaice,
                                land_spillover_class *land_spillover,
                                grid_class *grid,
                                int verbose);
void set_mask_bootstrap(bootstrap_class *this,
                        byte1 **loci, byte1 use_value);
bool get_seaice_bootstrap(bootstrap_class *this, 
                          byte2 **tbs[PMALGO_NUM_CHANNELS],
                          int row, int col, float *seaice);
bool bootstrap_scale_percent_seaice(bootstrap_class *this);
land_spillover_class *bootstrap_spillover(bootstrap_class *this);
bool bootstrap_recode_seaice( bootstrap_class *this,
                              grid_class *grid,
                              float **data,
                              int verbose );
void close_bootstrap(bootstrap_class *this);

#endif
