        subroutine read_hdf_land(hdf_filename, nx, ny, data_label, data_array, logfile_unit, status)

        implicit none

        !! this subroutine reads a int16 array from an HDF file

        !!  --------------- INPUT ----------------------
        !!      hdf_filename ........ name of the file to be read
        !!      nx .................. number of columns
        !!      ny .................. number of rows
        !!      data_label .......... HDF sds label of data to be read
        !!      data_array .......... data to be read
        !!      logfile_unit ........ log file unit
        !!
        !!  --------------- OUTPUT ----------------------               
        !!  status .............. will return -1 if something fails, else 0
                
        character*240 hdf_filename, data_label
        integer*4 nx, ny
        integer*2 data_array(nx, ny)
        
        integer*4 sd_id, sds_id, rank
        integer*4 dims(2), start(2), edges(2), stride(2)
        integer*4 sfstart, sfselect, sfrdata, sfendacc, sfend
        
        integer*4 dfacc_read, fail, istat
        parameter (dfacc_read = 1, fail = -1)
        
        integer*4 sd_index
        
        integer*4 logfile_unit
        
        integer*2 status
        
        rank=2
        dims(1)=nx
        dims(2)=ny
        
        status=fail
        
        sd_id = sfstart(hdf_filename, dfacc_read)
        if (sd_id .eq. FAIL) goto 999
        
        sd_index = 0 
        sds_id = sfselect(sd_id, sd_index)
        if (sds_id .eq. FAIL) goto 999
        
        start(1)=0
        start(2)=0
        edges(1)=nx
        edges(2)=ny
        stride(1)=1
        stride(2)=1
        
        istat = sfrdata(sds_id, start, stride, edges, data_array)
        if (istat .eq. FAIL) goto 999

        istat = sfendacc(sds_id)
        if (istat .eq. FAIL) goto 999

        istat = sfend(sd_id)
        if (istat .eq. FAIL) goto 999

        status=0
        
999     continue

        if (status .ne. 0) then
          write(logfile_unit,*) 'read_hdf_land failed:  '//hdf_filename
          stop 1
        endif
        
        end
