c==============================================================================
c seaice_bootstrap_v02 - provides an interface between pmalgos and the
c                        GSFC-provided bootstrap version 2 code.
c
c Input:
c    nr - number of rows in the grid.
c    nc - number of columns in the grid.
c    iv37 - 37V brightness temperatures x 10.
c    iv19 - 19V brightness temperatures x 10.
c    ih37 - 37H brightness temperatures x 10.
c    iv22 - 22V brightness temperatures x 10.
c    iiceout - sea ice concentrations.
c    landval - value used to designate land.
c    missval - value used to designate missing.
c    land_byte - land mask.
c    imonth - month of the data being processed (1-12).
c    iday - day of month of the data being processed (1-31).
c    hem - hemisphere code (1 => north, 2 => south).
c    satellite - bootstrap satellite name
c    verbose - verbosity level.
c
c 02/04/1998 Smith smithj@nsidc.org 303-492-5692
c National Snow & Ice Data Center, University of Colorado, Boulder
c Copyright (C) 2010 Regents of the University of Colorado
c==============================================================================
      subroutine seaice_bootstrap_v02(nr, nc, iv37, iv19, ih37,
     >     iv22, iiceout, landval, missval, land_byte, imonth, iday,
     >     hem, satellite, verbose)

      integer*4 nr, nc
      integer*2 iv37(nc,nr), iv19(nc,nr), ih37(nc,nr), iv22(nc,nr),
     >     iiceout(nc,nr), imonth, iday, hem, verbose
      byte land_byte(nc,nr)
      real*4 landval,missval

      real*4 v37(nc,nr), v19(nc,nr), h37(nc,nr), v22(nc,nr),
     >     iceout(nc,nr)
      integer*2 land(nc,nr)
      character*2 month, day
      character*9 satellite
      real*4 minval, maxval


      minval = 0.0
      maxval = 100.0

c
c Brightness temperatures are stored as Kelvinx10 and need to be converted to
c Kelvin. The land mask needs to be converted to a 2-byte integer.
c
      do i=1,nr
         do j=1,nc
            v37(j,i) = 0.1*FLOAT(iv37(j,i))
            v19(j,i) = 0.1*FLOAT(iv19(j,i))
            h37(j,i) = 0.1*FLOAT(ih37(j,i))
            v22(j,i) = 0.1*FLOAT(iv22(j,i))
            land(j,i) = land_byte(j,i)
         enddo
      enddo
      write(month, '(i2.2)') imonth
      write(day, '(i2.2)') iday

      if (hem .eq. 1) then
         call boot_ice_nsba(nc, nr, v37, v19, h37, v22, iceout,
     >        landval, missval, land, month, day, satellite)
      else if (hem .eq. 2) then
         call boot_ice_ssba(nc, nr, v37, v19, h37, v22, iceout,
     >        landval, missval, land, month, satellite)
      endif

      call fix_output_gdprod(iceout, iiceout, nc, nr, minval, maxval,
     >     landval, missval)

      return
      end
