      program comiso_algo
c+
c   This program opens the file produced by ice_con.pro that contains information
c   to run the comiso algorithm.  The comiso algorithm is called in this program
c   (comiso_sub) using the information stored in the IDL-produced file.
c   Julienne Stroeve 11/21/96.
c
c   Add code to accept the flag of satellite option
c   Xiaoming Li 10/10/97
c-
      character*6  date
      character*1  nors
      character*10 tb19v_path, tb22v_path, tb37h_path, tb37v_path
      character*19 ice_path
      character*3  icetype
      character*3  overlap
      integer      inors
      call getarg(1, overlap)
      open(unit=1,file='ice_con.dat',status='old')
      read(1,2)nors,date,tb19v_path,tb22v_path,tb37h_path,tb37v_path
 2    format(a1,a6,4a10)
      if(nors .eq. '1')inors = 1
      if(nors .eq. '2')inors = 2
      call comiso_sub(overlap, date,inors,tb19v_path,tb22v_path,
     +tb37h_path,tb37v_path)
      close(1)
      stop
      end
C
C  PROGRAM TO DETERMINE SEA ICE CONCENTRATION IN POLAR  REGIONS.
C
C                         PROGRAM ICCONC
C
C                   DEVELOPED FOR J. C. COMISO    NASA
C                  BY S.E.SCHWEINFURTH S.A.R.  9-25-85
C
C  THIS PROGRAM CALCULATES SSM/I ICE CONCENTRATION FROM 4 CHANNELS (19V,22V,
C  37V, AND 37H) FOR THE ARCTIC AND 3 CHANNELS (19V,22V AND 37V) FOR THE ANTARCTIC.
C
C
C  VARIABLES:
C      V37 = 316-BY-1 ELEMENT ARRAY CONTAINING A RECORD OF TB'S
C             FROM THE FIRST (37V) CHANNEL.
C      H37 = 316-BY-1 ELEMENT ARRAY CONTAINING A RECORD OF TB'S
C             FROM THE SECOND (37H) CHANNEL.            
C      V19 = 316-BY-1 ELEMENT ARRAY CONTAINING A RECORD OF TB'S
C             FROM THE THIRD (19V) CHANNEL.
C      V22 = 316-BY-1 ELEMENT ARRAY CONTAINING A RECORD OF TB'S
C             FROM THE FORTH (22V) CHANNEL.
C      OUT1 = 316-BY-1 ELEMENT ARRAY CONTAINING A RECORD OF CALCULATED
C             ICE CONCENTRATIONS.
C      MASK = 316-BY-1 ELEMENT ARRAY CONTAINING A LAND MASK RECORD.
C      IROW = ROW NUMBER FOR DATA MATRIX (332X316 SP OR 448X304 NP).
C      ICOL = COL NUMBER FOR DATA MATRIX (332X316 SP OR 448X304 NP).
C      WTP(1) = WATER TIE POINT OF FIRST CHANNEL, 37V
C      WTP(2) = WATER TIE POINT OF SECOND CHANNEL, 37H
C      WTP(3) = WATER TIE POINT OF THIRD CHANNEL, 19V
C      ITP(1) = ICE TIE POINT OF FIRST CHANNEL, 37V
C      ITP(2) = ICE TIE POINT OF SECOND CHANNEL, 37H
C      ITP(3) = ICE TIE POINT OF THIRD CHANNEL, 19V
C      VH37(1) = INTERCEPT FOR CONSOLIDATED ICE LINE, VH37
C      VH37(2) = SLOPE FOR CONSOLIDATED ICE LINE, VH37
C      V1937(1) = INTERCEPT FOR CONSOLIDATED ICE LINE, V1937
C      V1937(2) = SLOPE FOR CONSOLIDATED ICE LINE, V1937
C      WINTRC,WSLOPE = INTERCEPT AND SLOPE FOR EXCLUSION OF WATER
C                      DATA (BASED ON V1937 DATA)
C      WXLIMT = PARAMETER FOR EXCLUSION OF WATER DATA (BASED ON V19 and V22)
C      RADT = RADIAL DISTANCE BETWEEN WATER AND ICE TIE POINTS 
C      TTPI,TTPS = INTERCEPT AND SLOPE OF LINE BETWEEN
C                  WATER AND ICE TIE POINTS (V1937)               
C      WTRK = LOWEST PERCENTAGE FOR WATER DATA (SET AT 0-7.99%).
C      TICEK = ADDITIONAL FACTOR USED IN THE ADJUSTMENT OF BRIGHTNESS
C              TEMPERATURES FROM THEIR SAVED VALUES TO ACTUAL VALUES.
C      NORS = NP OR SP INDEX ( 1 FOR NP, 2 FOR SP)
C      SEASON = SEASON INDEX ( 1 FOR WARM SEASON, 2 FOR COLD SEASON)
C      PL = NUMBER OF LAND PIXELS
C      P0 = NUMBER OF BAD OR MISSING DATA PIXELS
C      PW = NUMBER OF WATER PIXELS
C      PWH = NUMBER OF WEATHER FILTERED PIXELS
C      RADCNT = NUMBER OF PIXELS WHOSE ICE CONCENTRATION WAS CALCULATED
C               VIA THE V1937 RADIUS METHOD.
C      ADCNT = NUMBER OF PIXELS WHOSE ICE CONCENTRATION WAS CALCULATED
C              VIA AN ADDITIONAL METHOD (VH37 INTERSECTION METHOD). 
C      NORMCNT = NUMBER OF PIXELS WHOSE ICE CONCENTRATION WAS CALCULATED
C                VIA THE V1937 INTERSECTION METHOD.     
C
C----------DECLARE VARIABLES
C
c**************************************************************************************
       subroutine comiso_sub(overlap, date,inors,tb19v_path,tb22v_path,
     + tb37h_path,tb37v_path)
c**************************************************************************************
       DIMENSION V37(316),H37(316),V19(316),V22(316),MASK(316)
     > ,WTP(3),ITP(3),VH37(2),V1937(2),OUT1(316)
       INTEGER*2 OUT1,V37,H37,V19,V22,MASK,F11,NORS,SEASON
       CHARACTER*80 OFILN,FILEN37V,FILEN37H,FILEN19V,FILEN22V
       CHARACTER*2 MONTH,DAY,YEAR
       CHARACTER*1 POLE
       DOUBLE PRECISION  INTRCA,SLOPEB,X(15000),Y(15000),SIGMAYY
     > ,SIGMAA,SIGMAB,R,SIGMAY(15000)
       LOGICAL*4 TEST
       REAL*4 WTP,ITP,VH37,V1937,WINTRC,WSLOPE,WXLIMT
     > ,RADT,TTPI,TTPS,WTRK,SLOPE,XINT1,YINT1,OFFSET
     > ,XINT2,YINT2
       INTEGER * 4 IROW,ICOL
     > ,PW,PL,P0,ICNT
c declarations added to work with the IDL driver routine  J. Stroeve 11/21/96
       integer h_ncols, q_ncols
       character*6  date
       character*3  overlap
       character*10 tb19v_path, tb22v_path, tb37h_path, tb37v_path
       character*10 maskfile, ice_path
       integer*2    hword
       byte         output(316),temp(2)
       equivalence  (temp, hword)
C
C  SET UNIVERSAL PARAMETERS
C
       DATA TEST/.FALSE./
       WTRK=0.0
       TICEK=1.0
       nors=inors
      write(6,*)'date = ',date
      write(6,*)'nors = ',nors
      write(6,*)'tb19v_path = ',tb19v_path
      write(6,*)'tb22v_path = ',tb22v_path
      write(6,*)'tb37h_path = ',tb37h_path
      write(6,*)'tb37v_path = ',tb37v_path
C
       IF (NORS .EQ. 1) THEN
          NORS = 1 
          POLE='N'
          NCOLS=304
          NROWS=448
          h_ncols=ncols/2
          q_ncols=ncols/4
          maskfile = 'GSFC_N.LOW'
          ice_path = date//'.tot'
       ELSE IF (NORS .EQ. 2) THEN
          NORS = 2 
          POLE='S'
          NCOLS=316
          NROWS=332      
          h_ncols=ncols/2
          q_ncols=ncols/4
          ice_path = date//'.tot'
          maskfile = 'GSFC_S.LOW'
       ELSE
          GOTO 558
       ENDIF
       write(6,*)'ice_path = ',ice_path
       write(6,*)'maskfile = ',maskfile
C              
c+
c  Open output ice concentration file.
c-
      open(unit=21,file=ice_path,status='unknown',form='unformatted',
     +     access='direct',recl=q_ncols)
c
C
C  BEGIN PROCESSING LOOP
C
c       DO 201 while (.not. test)
C
C  IF USED, ZERO OUT THE X AND Y ARRAYS
C
       IF (NORS .EQ. 1) THEN
         DO I=1,15000
           X(I)=0.0
           Y(I)=0.0
           SIGMAY(I)=0.0
         END DO
       ENDIF
C
c+
c    Open brightness temperature files.
c-
      open(unit=11,file=tb37v_path,status='old',form='unformatted',
     +     readonly,recordtype='fixed',access='direct',recl=h_ncols)
      open(unit=12,file=tb19v_path,status='old',form='unformatted',
     +     readonly,recordtype='fixed',access='direct',recl=h_ncols)
      open(unit=13,file=tb22v_path,status='old',form='unformatted',
     +     readonly,recordtype='fixed',access='direct',recl=h_ncols)
      IF (nors .EQ. 1) THEN
      open(unit=14,file=tb37h_path,status='old',form='unformatted',
     +     readonly,recordtype='fixed',access='direct',recl=h_ncols)
      ENDIF

c Open mask file
      open(unit=10,file=maskfile,status='old',recordtype='fixed',
     +     form='unformatted',recl=h_ncols,readonly,access='direct')
C
C  EXTRACT YEAR,MONTH AND DAYFROM THE DATASET NAME. IF THE DATA USED IS  
C  AVERAGED DATA (EG: DAILY EXTENTION = .v37, WEEKLY AVERAGED = .v37wk)
C  THEN THE DAY IS AUTOMATICALLY SET TO 1.  
C
       YEAR = date(1:2)
       MONTH= date(3:4)
       DAY= date(5:6)
       READ(YEAR ,'(I2)') IYEAR 
       READ(MONTH,'(I2)') IMONTH
       READ(DAY,'(I2)') IDAY
       WRITE(6,*) POLE,IYEAR,IMONTH,IDAY
C
C  SET F8/F11/F13 AND SEASON BASED ON DATE
C
       IF (overlap .EQ. '') then
         IF(IYEAR .LE.91 ) THEN
              F11=0  
              F13=0
         ENDIF
         IF(IYEAR .EQ. 91 .AND. IMON.EQ.12 .AND. IDAY.GE.3) THEN
               F11 = 1
               F13=0
         ENDIF
         IF(IYEAR .GE.92 ) THEN
               F11=1 
               F13=0
         ENDIF
         IF(IYEAR .EQ.95 .AND. IMON.LE.4) THEN
               F11=1
               F13=0
          ENDIF
         IF(IYEAR .EQ. 95 .AND. IMON.EQ.5 .AND. IDAY.GE.3) THEN
               F11=0
               F13=1
         ENDIF
         IF(IYEAR .GE.96) THEN
            F11=0
            F13=1
         ENDIF
       ELSE
         F11 = 0
         F13 = 0
         IF (overlap .eq. 'F11') THEN
                F11 = 1
         ELSE IF (overlap .EQ. 'F13') THEN
                F13=1
         ELSE IF (overlap .NE. 'F08' .AND.  overlap .NE. 'F8'  .AND. 
     >       overlap .NE. 'F13') THEN
                  WRITE(*,*) 'Argument ', overlap, ' unknown'
                  RETURN
         ENDIF
       ENDIF
       IF(NORS .EQ. 1 .AND. IMONTH .GE. 6 .AND. IMONTH .LE. 10)
     > SEASON=1
       IF(NORS .EQ. 1 .AND. IMONTH .EQ. 10 .AND. IDAY .GE. 16)
     > SEASON=2
       IF(NORS .EQ. 1 .AND. (IMONTH .GE. 11 .OR. IMONTH .LE. 5))   
     > SEASON=2
       IF(NORS .EQ. 2 .AND. (IMONTH .GE. 11 .OR. IMONTH .LE. 1))    
     > SEASON=1
       IF(NORS .EQ. 2 .AND. (IMONTH .GE.  2 .AND. IMONTH .LE. 10))    
     > SEASON=2
       WRITE(6,*) 'NORS=',NORS,' SEASON=',SEASON
C
C  SET PARAMETERS ACCORDING TO POLE AND SEASON OR TIME OF YEAR.
C  FOR REFERENCE, SEE COMISO, JGR(1986)
C  ARTICLE.
C
C
       IF (NORS .EQ. 1) THEN
         IF (SEASON.EQ.1) THEN
           WTP(1)= 203.0
           WTP(2)= 130.0
           WTP(3)= 181.0
           ITP(1)= 267.0
           ITP(2)= 259.0
           ITP(3)= 270.0
           VH37(1)= -12.00
           VH37(2)= 1.0000
           V1937(1)= 145.000
           V1937(2)= 0.467 
           WINTRC= 72.260
           WSLOPE= 0.5800
           WXLIMT= 23
         ELSE
           WTP(1)= 202.0
           WTP(2)= 130.0
           WTP(3)= 179.0
           ITP(1)= 258.0
           ITP(2)= 249.0
           ITP(3)= 263.0
           VH37(1)= -12.000
           VH37(2)= 1.0000
           V1937(1)= 117.000
           V1937(2)= 0.5533
           WINTRC= 78.000
           wslope=0.5667
           WXLIMT= 14
         ENDIF
C        
         IF(IMONTH .EQ. 5 .AND. IDAY .GE. 15) THEN
           V1937(1)=119
           V1937(2)=0.56
         ENDIF
C        
         IF(IMONTH .EQ. 6) THEN
           VH37(1)= -12.0000
           VH37(2)=1.0000          
           V1937(1)=119
           V1937(2)=0.56
           IF(IDAY .LE.23) THEN
             WINTRC= 78.000
             WSLOPE=0.5667
           ENDIF
         ENDIF
C        
         IF(IMONTH .EQ. 7) THEN
           V1937(1)=119
           V1937(2)=0.56
           IF (IDAY .LE. 18) THEN
             VH37(1)= -70.06
             VH37(2)=1.2226
           ELSE
             VH37(1)= -25.00
             VH37(2)=1.0333
           ENDIF
         ENDIF
C        
         IF(IMONTH .EQ. 8) THEN
            V1937(1)=119
            V1937(2)=0.56
           IF (IDAY .LE.  4) THEN
             VH37(1)= -25.00
             VH37(2)=1.0333
           ELSE
             VH37(1)= -14.0000
             VH37(2)=0.9933
           ENDIF
         ENDIF
C    
         IF(IMONTH .EQ. 9) THEN
           IF (IDAY .LE. 12) THEN
             V1937(1)=119
             V1937(2)=0.56
           ELSE
             V1937(1)=103
             V1937(2)=0.6067
           ENDIF
           IF (IDAY .LE. 20) THEN
             VH37(1)= -14.0000
             VH37(2)= 0.9933
           ELSE
             VH37(1)= -12.0000
             VH37(2)= 1.0000
             WINTRC= 78.000
             WSLOPE= 0.5667 
           ENDIF 
         ENDIF
C        
         IF(IMONTH.EQ.10 ) THEN
           V1937(1)= 103
           V1937(2)= 0.6067 
           WINTRC= 78.000
           WSLOPE= 0.5667
         ENDIF
C        
         IF(IMONTH.EQ.11 .AND. IDAY .LE. 15) THEN
           V1937(1)= 103
           V1937(2)= 0.6067 
         ENDIF
       ENDIF
C
       IF(NORS .EQ. 2) THEN
         IF (SEASON.EQ.1) THEN
           WTP(1)= 203.0           
           WTP(3)= 180.0
           ITP(1)= 262.0
           ITP(3)= 256.0
           V1937(1)= 139.0
           V1937(2)= 0.4733
           VH37(1)= 0.0
           VH37(2)= 0.0
           WINTRC= 93.000 
           WSLOPE= 0.4933                           
           WXLIMT= 16
         ELSE     
           WTP(1)= 203.0           
           WTP(3)= 180.0
           ITP(1)= 262.0
           ITP(3)= 256.0
           V1937(1)= 139.0
           V1937(2)= 0.4733
           VH37(1)= 0.0
           VH37(2)= 0.0
           WINTRC= 93.000 
           WSLOPE= 0.4933      
           WXLIMT= 16
         ENDIF
         IF(IMONTH .EQ. 2 .OR. IMONTH .EQ. 3) THEN
           V1937(1)= 102
           V1937(2)= 0.6200
         ENDIF
         IF(IMONTH .EQ. 2 .AND. IDAY .LE. 7) THEN
           V1937(1)= 120.500
           V1937(2)= 0.5467
         ENDIF
         IF(IMONTH .EQ. 4 .AND.IDAY .LE. 7) THEN
           V1937(1)= 120.500
           V1937(2)= 0.5467
         ENDIF
       ENDIF
C
C  RESET COUNTS AND REWIND LAND MASK FILE
C
       PL=0        
       P0=0
       PW=0
       PWH=0
       RADCNT=0
       ADCNT=0
       NORMCNT=0      
       REWIND 10
C
C
C NEED TO CREATE 2D ARRAYS OF 37V AND 37H IN ORDER TO APPLY
C REGRESSION ANALYSIS FOR THE NEW VH37(1) AND VH37(2) VALUES.
C CURRENTLY THESE ARE JUST COMPUTED FOR THE SUMMER SEASON AND JUST FOR THE
C NP.  WE USE ALL THE POINTS WHICH ARE ABOVE THE LINE (Y=1.0X-22) FOR THE
C REGRESSION ANALYSIS.
C NOTE: 2 IS ADDED TO THE NEW INTERCEPT
C
C A. ALLEGRINO 4/26/95
C
C
       IF(NORS .EQ.1 .AND. SEASON .EQ. 1) THEN         
        ICNT=1
        DO IROW=1,NROWS
         READ(10) (MASK(I),I=1,NCOLS)
         READ(11) (V37(I),I=1,NCOLS)
         READ(14) (H37(I),I=1,NCOLS)
         DO ICOL= 1,NCOLS
           IF (MASK(ICOL).eq. 0) THEN
             TB1= (FLOAT (V37(ICOL)))/10.0
             TB2= (FLOAT (H37(ICOL)))/10.0
             IF(TB1 .GE. 40 .AND. TB2 .GE. 40 .AND. 
     >          TB1 .LT. 280 .AND. TB2 .LT. 280) THEN    
               IF(F11 .EQ. 1) TB1= (1.0 - 0.00335) *TB1 +0.01
               IF(F11 .EQ. 1) TB2= (1.0 - 0.00165) *TB2 +0.03
c added the F13 Wentz coefficients J. Stroeve 1/7/98
               IF(F13 .EQ. 1) THEN
                  TB1= (1.0 - 0.00335131)*TB1 - 0.49
                  TB2= (1.0 - 0.0016533)*TB2 - 0.05
               ENDIF
               C37H=TB1 -22
               IF(TB2 .GT. C37H) THEN
                 X(ICNT)=DBLE (TB1)
                 Y(ICNT)=DBLE (TB2)
                 ICNT=ICNT+1
                 IF(ICNT .GT. 14990) GO TO 8500
               ENDIF
             ENDIF 
           ENDIF
         END DO
        END DO
8500    CONTINUE
        CALL LINFIT(X,Y,SIGMAY,ICNT,0,INTRCA,SIGMAA,SLOPEB,SIGMAB,R,
     >   SIGMAYY)
        IF(SLOPEB .LE. 1.2226) THEN
          VH37(1)= INTRCA +2
          VH37(2)= SLOPEB
          write(6,*) 'used LINFIT:  ',VH37(1),VH37(2)
        ENDIF    
        REWIND(10)
        REWIND(11)
        REWIND(14)
       ENDIF        
C
C  DEFINE WATER-ICE LINE PARAMETERS: (1's are for VH37 
C                                     2's are for V1937)
C  AND RADIAL DISTANCE
C     
       TTPS=(ITP(3)-WTP(3))/(ITP(1)-WTP(1))
       TTPI=ITP(3)-TTPS*ITP(1)
       IF(NORS .EQ. 1) THEN 
          IF (SEASON .EQ. 1) THEN
            XINT1=266
            YINT1=270
          ELSE
            XINT1=261
            YINT1=263
          ENDIF
          SLOPE=(YINT1-WTP(3))/(XINT1-WTP(1))
          OFFSET= WTP(3) - (WTP(1)*SLOPE)
          XINT2=(OFFSET - V1937(1))/(V1937(2) - SLOPE)
          YINT2=(V1937(2) * XINT2) + V1937(1)
          RADT=SQRT( (XINT2-WTP(1))**2 + (YINT2-WTP(3))**2) 
       ELSE
          RADT=SQRT( (ITP(1)-WTP(1))**2 + (ITP(3)-WTP(3))**2)
       ENDIF 
C
C  PRINT THE VARIOUS STARTING PARAMETERS USED IN THE ALGORITHM
C
       WRITE(6,2) WTP
       WRITE(6,222) ITP
       WRITE(6,23) (VH37(J),J=1,2)
       WRITE(6,25) (V1937(J),J=1,2)
       WRITE(6,24) WINTRC,WSLOPE
       WRITE(6,111) RADT
       WRITE(6,29) TTPI,TTPS
       WRITE(6,27) WTRK
    2  FORMAT(' WATER TIEPOINTS ARE = ',3F8.2)
  222  FORMAT(' FIRST YEAR ICE TIE PTS = ',3F8.2)
   23  FORMAT(' THE VH37 CONSOLIDATED ICE LINE IS Y = ',F8.3,' + X*',
     > F8.3)
   25  FORMAT(' THE V1937 CONSOLIDATED ICE LINE IS Y = ',F8.3,' + X*',
     > F8.3)
   24  FORMAT(' THE WATER EQUATION IS Y = ',F8.3,' + X*',F8.3)
  111  FORMAT(' THE RADIUS IS ',F8.3)
   29  FORMAT(' THE LINE CONNECTING V1937 WATER TIEPTS WITH ICE TIEPTS IS:
     > Y = ',F8.3,' + X*',F8.3)
   27  FORMAT(' WATER CUT OFF AT ',F4.1,'%')
C
C  BEGIN PROCESSING DATA BY READING IN A RECORD OF MASK AND THREE TB'S
C
       DO 200 IROW = 1,NROWS
C
c read in a record of landmask and four Tb's
         READ(10) (MASK(I),I=1,NCOLS)
         READ(11) (V37(I),I=1,NCOLS)
         READ(12) (V19(I),I=1,NCOLS)
         READ(13) (V22(I),I=1,NCOLS)
         IF (NORS .EQ. 1) READ(14) (H37(I),I=1,NCOLS)
C         
         DO 150 ICOL = 1,NCOLS
C
C  TEMPERATURES LESS THAN 40K (400=40*10 (WHERE 10 IS OUR
C  TEMPERATURE SCALING FACTOR)) ARE TAGGED AS BAD DATA.  SKIP THESE AND ALL
C  LAND POINTS.
C
          IF(V37(ICOL).LT.400 .OR. V19(ICOL).LT.400 .OR.
     >    V22(ICOL).LT.400 .OR.
     >    MASK(ICOL).ne. 0) GOTO 50
          IF (NORS .EQ. 1 .AND. H37(ICOL).LT.400) GOTO 50
C         
          TB1=V37(ICOL)/(10.0*TICEK)
          IF (NORS .EQ. 1) THEN
            TB2=H37(ICOL)/(10.0*TICEK)
          ELSE 
            TB2=0.0
          ENDIF
          TB3=V19(ICOL)/(10.0*TICEK)
          TB4=V22(ICOL)/(10.0*TICEK)
C         
C***********************************************************************
C   F11 to F8 transformation provided by Frank Wentz thru Mike Martino 
C   A. Allegrino 3/5/96
C***********************************************************************
C
          IF(F11 .EQ. 1) THEN
           TB1= (1.0 - 0.00335) *TB1 +0.01
           TB2= (1.0 - 0.00165) *TB2 +0.03
           TB3= (1.0 - 0.00221) *TB3 -0.44
           TB4= (1.0 - 0.00161) *TB4 -0.30
          ENDIF
C***********************************************************************
C   F13 to F8 transformation provided by Frank Wentz
C***********************************************************************
C
          IF(F13 .EQ. 1) THEN
           TB1= (1.0 - 0.00335) *TB1 -0.49
           TB2= (1.0 - 0.00165) *TB2 -0.05
           TB3= (1.0 - 0.00221) *TB3 -0.04
           TB4= (1.0 - 0.00161) *TB4 -0.37
          ENDIF
C
C  CALL SUBROUTINE TO CALCULATE ICE CONCENTRATIONS.
C
           CALL ICCALC(NORS,SEASON,VH37,V1937,WTP,ITP,TB1,TB2,
     >     TB3,TB4,WSLOPE,WINTRC,RADT,TTPS,TTPI,WTRK,ITBD,
     >     ITBLW,IROW,ICOL,WXLIMT,PW,PWH,RADCNT,ADCNT,NORMCNT)
           OUT1(ICOL)=ITBD
c put in a statement to check for ice values over 100% and less than 0%.
c (Julienne Stroeve 5/9/97)
           IF (OUT1(ICOL) .GT. 1000) OUT1(ICOL) = 1000
           IF (OUT1(ICOL) .LT.    0) OUT1(ICOL) = 0
           GO TO 100
C
C  COME HERE IF THERE IS BAD OR MISSING DATA, OR IF PIXEL IS LAND
c  we changed this so that if there is bad or missing data, the pixel
c  values is set to (-99) 157  (Julienne Stroeve 11/21/96)
C  IF THE PIXEL IS LAND, THE PIXEL VALUE IS SET TO -800 AND THE LAND
C  PIXEL IS COUNTED.
c  we changed this so that if the pixel is land it is set to -88 (168)
c  Julienne Stroeve 11/21/96
c  these are the same changes than Nick installed before.
C
   50      CONTINUE
C
C  COUNT THE NUMBER OF BAD OR MISSING PIXELS
C
           IF(MASK(ICOL).eq.0) THEN
              OUT1(ICOL) = -990
              P0=P0+1
           ELSE
              OUT1(ICOL) = -880
              PL=PL+1
           ENDIF
  100      CONTINUE
C
C  WRITE ICE CONCENTRATIONS TO OUTPUT DEVICE.
C
c writing out byte values instead of integer (Julienne Stroeve 5/9/97)
c values are represented as single byte percent ranging from 0 - 100%.
       hword = nint(float(OUT1(icol)/10.))
       output(icol)=temp(2)
150    CONTINUE
c       WRITE(21) (OUT1(I),I=1,NCOLS)
       WRITE(21) (output(I),I=1,NCOLS)
  200  CONTINUE
       WRITE(6,210) PL,P0
       WRITE(6,213) PW,PWH
       WRITE(6,214) RADCNT,ADCNT,NORMCNT
  210  FORMAT(' # OF LAND PIXELS',I7,' BAD OR MISSING DATA PIXELS',I7)
  213  format(' # OF WATER FILTERED PIXELS',I7,' WEATHER PIX',I7)
  214  format(' # OF PIXELS BELOW WATER-ICE LINE',I7,' THAT USED VH37',I7,
     > ' THAT USED V1937',I7)
       CLOSE(11)
       CLOSE(12)
       CLOSE(13)
       IF (NORS .EQ. 1) CLOSE(14)
       CLOSE(21)
c201    CONTINUE
320    CONTINUE
       WRITE(6,*) 'NORMAL END OF PROGRAM.'
       STOP
  555  STOP555
  556  STOP556
  557  STOP557
  558  STOP558
       END
C***************************************************************
C                          SUBROUTINE ICCALC
C
C                    DEVELOPED FOR J. C. COMISO     NASA
C                     BY S.E.SCHWEINFURTH  S.A.R.  9-25-85
C
C  MODIFIED TO ACCOUNT FOR SSMI DATA BY A.ALLEGRINO 2/89.
C
C  THIS SUBROUTINE CALCULATES ICE CONCENTRATION BY
C  FINDING THE RATIO OF THE DISTANCES OF (WATER VALUE TO
C  DATA VALUE) TO (WATER VALUE TO POLYNOMIAL VALUE).
C  THE POLYNOMIAL VALUE IS FOUND BY INTERSECTING THE LINE
C  THROUGH WATER AND DATA POINTS WITH THE POLYNOMIAL.
C
C
C
C      VARIABLES:
C          IR,JC = ROW AND COLUMN #'S
C          TB1= 37V BRIGHTNESS TEMPERATURE
C          TB2= 37H BRIGHTNESS TEMPERATURE
C          TB3= 19V BRIGHTNESS TEMPERATURE
C          TB4= 22V BRIGHTNESS TEMPERATURE
C          WTP1= 37V WATER TIE POINT
C          WTP3= 37H WATER TIE POINT
C          WTP2= 19V WATER TIE POINT
C          ET1= RESULT FROM WATER EQUATION (THE CALCULATED 19V
C               BRIGHTNESS TEMPERATURE USED TO TEST FOR WATER PIXEL
C               (ALSO USED TO DETERMINE IF THE 19V DATA WOULD BE USED
C                FOR NP ICE CONCENTRATION CALCULATION)
C          ET2= VALUE OBTAINED FROM THE LINE BETWEEN THE CONSOLIDATED
C              ICE AND WATER TIE POINTS
C          EINT1= X-COORDINATE OF THE POINT OF INTERSECTION
C          EINT2= Y-COORDINATE OF THE POINT OF INTERSECTION
C          TTPS= SLOPE OF THE LINE BETWEEN THE CONSOLIDATED ICE TIE
C                POINT AND THE WATER TIE POINT (V1937)
C          TTPI= INTERCEPT OF THE LINE BETWEEN THE CONSOLIDATED ICE
C                AND THE WATER TIE POINTS (V1937)
C
C
       SUBROUTINE ICCALC(NORS,SEASON,VH37,V1937,WTP,ITP,TB1,TB2,
     > TB3,TB4,WSLOPE,WINTRC,RADT,TTPS,TTPI,WTRK,ITBD,ITBLW,
     > IR,JC,WXLIMT,PW,PWH,RADCNT,ADCNT,NORMCNT)
C
C------DECLARE VARIABLES
C
       INTEGER * 4 PW,ICNT,PWH,RADCNT,ADCNT,NORMCNT
       REAL*4 VH37(2),WTP(3),TB1,TB2,TB3,TB4,ICEC,ET1,ET2
     > ,ET19V,WSLOPE,WINTRC,V1937(2),ET3,SL37,ET37H
     > ,RADT,TTPS,TTPI,ITP(3),WTRK
       INTEGER * 2 NORS,SEASON
C
C  SET UP EQUATIONS AND INITIALIZE VARIABLES.
C  DATA BELOW THE OPEN WATER LINE (FILTER) IS ASSUMED TO BE LESS THAN
C  8%.
C
       WTP1=WTP(1)
       WTP2=WTP(2)
       WTP3=WTP(3)
C
C  THIS IS A NEW WATER FILTER USING THE 19V VS 22V SCATERPLOT
C  TB4 IS THE 22V TEMPERATURE
C
C  A. ALLEGRINO 5/25/95
C
       ET1 = WSLOPE * TB4 + WINTRC
       IF(ET1.GT.TB3)THEN
        ICEC=WTRK/100.
        PW=PW+1
        GOTO 75
       ENDIF
C
C  APPLY ATMOSPHERIC FILTER. THE (22V-19V) TEMPERATURE DIFFERENCE IS
C  COMPARED WITH A CERTAIN VALUE . IF THE DIFFERENCE > THIS VALUE THEN THE PIXEL
C  IS SAID TO CONTAIN WEATHER AND IS CONVERTED TO THE VALUE FOR OPEN
C  OCEAN. THE VALUE CHANGES WITH HEMISPHERE AND SEASON, AND WAS DERIVED EMPIRICALLY.
C
       ACTDIFF=TB4-TB3
       IF(ACTDIFF .GT. WXLIMT)THEN
        ICEC=WTRK/100.
        PWH=PWH+1
        GOTO 75
       ENDIF
C
C  THERE ARE THREE OPTIONS CONTAINED IN THE FOLLOWING IF
C  STATEMENT.  THE FIRST FILTERS OUT THOSE POINTS BELOW
C  THE VH1937 WATER-ICE LINE AND USES THE V1937 PLOT WITH RADT 
C  TO CALCULATE CONCENTRATIONS.  THE SECOND OPTION IS FOR THOSE 
C  POINTS ABOVE A SET CRITERIA (AD-5) IN VH37 SPACE (NP ONLY) AND
C  USES VH37 SPACE. THE THIRD (DEFALUT) USES V1937 SPACE.
C  
       ET19V=TTPI+TTPS*TB1
       ET3=( VH37(1)-5 )+( VH37(2)*TB1 )
       IF (TB3 .LE. ET19V) THEN
         RADCNT=RADCNT+1
         R1=SQRT( (TB1-WTP1)**2 + (TB3-WTP3)**2 )
         ICEC=R1/RADT
       ELSE IF (NORS .EQ. 1 .AND. TB2 .GT. ET3) THEN
         ADCNT=ADCNT+1
         ED=TB1-WTP1
         IF(ED .EQ. 0.0)THEN
           EINT1=WTP1
         ELSE
           BET=(TB2-WTP2)/ED
           ALP=TB2-(BET*TB1)
           SLDVH=VH37(2)-BET
           IF(SLDVH .EQ. 0.0) THEN
             ICEC=WTRK/100.
             GOTO 75
           ELSE
             EINT1=(ALP-VH37(1))/SLDVH
           ENDIF
         ENDIF
         EINT2=VH37(1)+(VH37(2)*EINT1)      
         RINT=SQRT( (EINT1-WTP1)**2 + (EINT2-WTP2)**2 )
         IF (RINT .EQ. 0)THEN
            ICEC=WTRK/100.
         ELSE
            R1=SQRT( (TB1-WTP1)**2 + (TB2-WTP2)**2 ) 
            ICEC=R1/RINT
         ENDIF 
       ELSE
         NORMCNT=NORMCNT+1
         ED=TB1-WTP1
         IF(ED .EQ. 0.0)THEN
           EINT1=WTP1
         ELSE
           BET=(TB3-WTP3)/ED
           ALP=TB3-(BET*TB1)
           SLDVH=V1937(2)-BET
           IF(SLDVH .EQ. 0.0) THEN
             ICEC=WTRK/100.
             GOTO 75
           ELSE
             EINT1=(ALP-V1937(1))/SLDVH
           ENDIF
         ENDIF
         EINT2=V1937(1)+(V1937(2)*EINT1)      
         RINT=SQRT( (EINT1-WTP1)**2 + (EINT2-WTP3)**2 )
         IF (RINT .EQ. 0)THEN
            ICEC=WTRK/100.
         ELSE
            R1=SQRT( (TB1-WTP1)**2 + (TB3-WTP3)**2 ) 
            ICEC=R1/RINT
         ENDIF
       ENDIF                  
C
C  SET ICE CONCENTRATIONS LIMITS FOR NASA COLOR PROGRAM.
C  ICE CONCENTRATION UP TO 108% ARE STORED BECAUSE OF UNCERTAINTIES
C  IN THE EMISSIVITY OF CONSOLIDATED ICE.
C
75     ICEC = ICEC * 100.0
       IF(ICEC.LT.WTRK) ICEC = WTRK
       IF(ICEC.GT.108.0) ICEC = 108.0
C
C  WE PACK THE ICE CONCENTRATIONS IN THE FOLLOWING MANNER.
C  THIS IS JUST A SCALING TO BE ABLE TO USE NASA COLOR SOFTWARE
C  CONVENIENTLY.  AN ALTERNATIVE WOULD BE TO MULTIPLY ICEC BY 10
C  BEFORE STORING IT AS INTEGER.
C
       ITBD=INT((ICEC*10)+0.5)
       RETURN
       END

c********************************************************************
c
c  This linfit routine is from Bevington's Statical book      
c
      SUBROUTINE LINFIT(X,Y,SIGMAY,NPTS,MODE,A,SIGMAA,B,SIGMAB,R,
     >sigmayy)       
      DOUBLE PRECISION SUM,SUMX,SUMY,SUMX2,SUMXY,SUMY2             
      DOUBLE PRECISION XI,YI,WEIGHT,DELTA,VARNCE             
      double precision x,y,sigmay,a,sigmaa,b,sigmab,r,sigmayy
      DIMENSION X(1),Y(1),SIGMAY(1)                                     
C                                                                   
C                                                                      
   11 SUM = 0.                                                      
      SUMX = 0.                                                        
      SUMY = 0.                                                        
      SUMX2 = 0.                                                       
      SUMXY = 0.                                                       
      SUMY2 = 0.                                                       
   21 DO 50 I=1,NPTS                                                 
      XI = X(I)                                                       
      YI = Y(I)                                                       
      IF(MODE) 31,36,38                                               
   31 IF(YI) 34,36,32                                                  
   32 WEIGHT = 1.0/YI                                                 
      GO TO 41                                                        
   34 WEIGHT = 1.0/(-YI)                                              
       GO TO 41                                                       
   36 WEIGHT = 1.0                                                    
      GO TO 41                                                        
   38 WEIGHT = 1.0/SIGMAY(I)**2                                       
   41 SUM = SUM + WEIGHT                                              
      SUMX = SUMX + WEIGHT*XI                                         
      SUMY = SUMY + WEIGHT*YI                                         
      SUMX2 = SUMX2 + WEIGHT*XI*XI                                    
      SUMXY = SUMXY + WEIGHT*XI*YI                                    
      SUMY2 = SUMY2 + WEIGHT*YI*YI                                    
   50 CONTINUE                                                         
   51 DELTA = SUM*SUMX2 - SUMX*SUMX                                   
      A = (SUMX2*SUMY - SUMX*SUMXY)/DELTA                             
   53 B = (SUMXY*SUM - SUMX*SUMY)/DELTA                               
   61 IF(MODE) 62,64,62                                               
   62 VARNCE = 1                                                      
      GO TO 67                                                        
   64 C = NPTS - 2                                                    
      VARNCE = (SUMY2 + A*A*SUM + B*B*SUMX2                           
     1 - 2.0*(A*SUMY + B*SUMXY - A*B*SUMX))/C                         
      sigmayy=dsqrt(varnce)
   67 SIGMAA = DSQRT(VARNCE*SUMX2/DELTA)                              
   68 SIGMAB = DSQRT(VARNCE*SUM/DELTA)                                
   71 R = (SUM*SUMXY - SUMX*SUMY)/                                    
     1 DSQRT(DELTA*(SUM*SUMY2 - SUMY*SUMY))                           
      RETURN                                                           
      END
