/*======================================================================
 * goddard - storage and retrieval for ancillary information needed
 *           to run the Goddard time series sea ice algorithm.
 *
 * 05/24/2006 brodzik brodzik@nsidc.org 303-492-8263
 * National Snow & Ice Data Center, University of Colorado, Boulder
 * Copyright (C) 2006 University of Colorado at Boulder
 *
 *$Revision: 1.2.6.1 $
 *
 *======================================================================*/
#ifndef goddard_h_
#define goddard_h_

#include "define.h"
#include "pmalgorithm.h"
#include "land_spillover.h"

static const char goddard_h_rcsid[] = "$Header: /tmp_mnt/FILES/pmalgos/goddard.h,v 1.2.6.1 2008-06-10 20:36:35 brodzik Exp $";

/* Structure for ancillary algorithm information */
typedef struct {
    pmalgo_hemisphere hem;                /* Hemisphere id */
    pmalgo_ssmi_platform platform;        /* SSM/I platform id */
    bool scale_percent_seaice;            /* flag for scaling seaice values to 0-100% */
    land_spillover_class *land_spillover; /* flag for doing spillover correction */
    int goddard_recoding_pcent;           /* minimum seaice threshold for goddard recoding */
    int verbose;                          /* verbosity flag */
} goddard_class;
  
/* Function prototypes */
bool is_goddard_algorithm(pmalgo_id algorithm);

goddard_class *init_goddard(const int goddard_recoding_pcent,
                            const char *hemisphere_name,
			    pmalgo_ssmi_platform platform,
                            const bool scale_percent_seaice,
			    land_spillover_class *land_spillover,
                            grid_class *grid,
			    int verbose);

pmalgo_hemisphere goddard_hemisphere(goddard_class *this);

pmalgo_ssmi_platform goddard_platform(goddard_class *this);

bool goddard_scale_percent_seaice(goddard_class *this);

land_spillover_class *goddard_spillover(goddard_class *this);

bool goddard_recode_seaice( goddard_class *this,
                            grid_class *grid,
                            float **data,
                            bool use_climatology,
                            byte1 **climatology,
                            bool use_loci,
                            byte1 **loci,
                            bool fill_algo_pixels_only,
                            bool do_user_fill_value,
                            float fill_value,
                            int verbose );

void close_goddard(goddard_class *this);

#endif
