/*======================================================================
 * fortran.h - C calling FORTRAN stuff
 * 
 * Acknowledgements:  Thanks to jthorn for the idea of encapsulating
 *                    the mangling schema here.
 *
 * 02/03/2006 brodzik brodzik@nsidc.org 303-492-8263
 * National Snow & Ice Data Center, Boulder, CO
 * Copyright (C) 2006 University of Colorado at Boulder
 *
 * $Id: fortran.h 17946 2011-01-19 17:26:24Z smallory $
 *
 *======================================================================*/

#ifndef FORTRAN_H_SEEN
#define FORTRAN_H_SEEN

/*
 * Names of Fortran routines are often altered by the compiler/loader.  The
 * following macro should be used to call a Fortran routine from C code, i.e.
 *	call sgefa(...)			-- Fortran code
 *	FORTRAN_NAME(sgefa)(...);	-- C code to do the same thing
 *
 * Unfortunately, the "alterations" are generally at the token level, and this
 * can't be done portably in pre-ANSI C.  In ANSI C, the preprocessor "token
 * splicing" facility is designed to handle just this sort of thing, but in
 * pre-ANSI C we have to use rather ugly system-dependent hacks of the sort
 * exemplified below.
 */

#if defined(sgi) || defined(SGI) || defined(__sgi__) || defined(__SGI__)
  /* C code should reference Fortran names in lower case */
  #ifdef __STDC__
    #define FORTRAN_NAME(name)  name ## _
  #else
    #define FORTRAN_NAME(name)  name/**/_
  #endif

#elif defined(GFORTRAN)
/* C code should reference Fortran names in lower case */
/* With the change to gfortran, gcc has reverted to always */
/* append a single underscore. This preceeds the GNUC option */
/* so that __GNUC__ does not override the presence of GFORTRAN. */
  #ifdef __STDC__
    #define FORTRAN_NAME(name)	name ## _
  #else
    #define FORTRAN_NAME(name)	name/**/_
  #endif

#elif defined(GFORTRAN)
/* C code should reference Fortran names in lower case */
/* With the change to gfortran, gcc has reverted to always */
/* append a single underscore. This preceeds the GNUC option */
/* so that __GNUC__ does not override the presence of GFORTRAN. */
  #ifdef __STDC__
    #define FORTRAN_NAME(name)	name ## _
  #else
    #define FORTRAN_NAME(name)	name/**/_
  #endif

#elif defined(GNUC) || defined(__GNUC__)
/* C code should reference Fortran names in lower case */
/* Note: appending 2 underscores is only appropriate for original */
/* names with an underscore; if original name has no underscore, */
/* gcc appends a single underscore, so a new macro will need to */
/* be defined */
  #ifdef __STDC__
    #define FORTRAN_NAME(name)	name ## __
  #else
    #define FORTRAN_NAME(name)	name/**/__
  #endif

#endif

#endif	/* FORTRAN_H_SEEN */
