/*=============================================================================
 * Header file for land spillover correction routines. These routines were
 * separated from goddard.c on 2/11/2010.
 *
 * National Snow and Ice Data Center, University of Colorado, Boulder
 * Copyright 2010 Regents of the University of Colorado
 *==========================================================================*/

#ifndef land_spillover_h_
#define land_spillover_h_

#include "define.h"
#include "grids.h"

typedef struct {
    int spillover_open_water_pcent;
    int bootstrap_recoding_pcent;
    byte1 **M;
    byte1 **CMIN;
    char *se_path;
} land_spillover_class;

land_spillover_class *init_land_spillover(const int spillover_open_water_pcent,
					  const int algorithm_open_water_pcent,
					  const char *landmask_filename,
					  const char *weights_filename,
					  grid_class *grid,
					  int verbose);
static char *morph_se_path( int verbose );
static bool make_proximity_mask(land_spillover_class *this,
				const char *landmask_filename,
				grid_class *grid,
				int verbose);
static bool make_CMIN_weights(land_spillover_class *this,
			      const char *weights_filename,
			      grid_class *grid,
			      int verbose);
bool spillover_seaice( land_spillover_class *this,
                       bool scale_percent_seaice,
                       float spillover_open_water_pcent,
                       float scaling_factor,
                       int max_ice_percent,
                       grid_class *grid,
                       float **data,
                       int verbose);

#endif
