/* dilatehs.c */
/*                                                  */
/* includes: main */

/* depends on: ReadRasterFile, WriteRasterFile, MakeColorMap */

/* version 1.0    25 August 1996 */

/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */ 
/* Nashville, TN 37235                             */ 
/* rap2@vuse.vanderbilt.edu                        */ 
/*                                                 */ 
/* This software is freely redistributable if      */ 
/* the author's name and affiliation are included. */

#include "stdio.h"
#include "malloc.h"
#include "rasterio.h"
#include "math.h"

#define CMAPSZ 768
#define ALLOC    1
#define WHITE  255
#define BLACK    0
#define HPI    128
#define H2PI   256
#define ZERO     0
#define SEPATH "SEPATH"
#define LUM      1
#define HUE      2
#define SAT      3

int *LoadSE();
char *getenv();
void MapThroughMap();
void LumSetDilate();
void HueSetDilate();
void SatSetDilate();

main( argc, argv )
   unsigned int argc;
   char * argv[];
   {
   byte *ImgL,*DilL;
   byte *ImgH,*DilH;
   byte *ImgS,*DilS;
   struct rasterfile RasHdL;	/* raster file header */
   struct rasterfile RasHdH;	/* raster file header */
   struct rasterfile RasHdS;	/* raster file header */

   int X;
   int Y;
   int i,n;
   byte pix;
   char *InNameLum, *OutNameLum;
   char *InNameHue, *OutNameHue;
   char *InNameSat, *OutNameSat;
   char *SEName;
   FILE *fpil, *fpol;
   FILE *fpih, *fpoh;
   FILE *fpis, *fpos;
   byte *CMapL,*CMapH,*CMapS,*MapGS;
   byte *CL,*CH,*CS;
   byte *IL,*IH,*IS;
   byte *u,*v,*w;
   int  *SE;
   int sX, sY, sorgx, sorgy;
   int ulx, uly;              /* upper left  hand offset for zero padding */
   int lrx, lry;              /* lower right hand offset for zero padding */
   int N;
   int setfun;
   int optype;

   /* check args */

   if ( argc < 2 )
      {
      fprintf(stderr, "usage: dilateluv -l|h|s LumIn HueIn SatIn LumOut HueOut SatOut SEFile\n");
      exit(0);
      }


   /* get args */
   fprintf( stderr,"getting arguments\n");
   InNameLum  = argv[argc-7];
   InNameHue  = argv[argc-6];
   InNameSat  = argv[argc-5];
   OutNameLum = argv[argc-4];
   OutNameHue = argv[argc-3];
   OutNameSat = argv[argc-2];
   SEName     = argv[argc-1];

   /* get lum | hue | sat switch */
   optype = LUM;
   if ( argv[1][0] == '-' )
      {
      if ( argv[1][1] == 'l' )
         optype = LUM;
      else if ( argv[1][1] == 'h' )
         optype = HUE;
      else if ( argv[1][1] == 's' )
         optype = SAT;
      else
         {
         fprintf( stderr, "Illegal argument\n");
         exit( 0 );
         }
      }

   /* load the structuring element */
   fprintf( stderr,"Loading SE\n");
   SE = LoadSE( SEName, &sX, &sY, &sorgx, &sorgy );

   /* compute zero-pad dimensions */
   fprintf( stderr,"computing dimensions\n");
   ulx = sX - sorgx - 1;
   uly = sY - sorgy - 1;
   lrx = sorgx;
   lry = sorgy;

   /* open the image input files */
   fprintf( stderr,"opening files\n");
   if ( !(fpil = OpenFile( InNameLum, NULL, "r" )) ) exit( 0 );
   if ( !(fpih = OpenFile( InNameHue, NULL, "r" )) ) exit( 0 );
   if ( !(fpis = OpenFile( InNameSat, NULL, "r" )) ) exit( 0 );


   /* read the input images into memory */
   fprintf( stderr,"reading images\n");
   if ( ReadRasterFile( fpil, &RasHdL, &CMapL, &ImgL, ulx, uly, lrx, lry, ALLOC ) ) 
      exit(0);
   fclose( fpil );
   if ( ReadRasterFile( fpih, &RasHdH, &CMapH, &ImgH, ulx, uly, lrx, lry, ALLOC ) ) 
      exit(0);
   fclose( fpih );
   if ( ReadRasterFile( fpis, &RasHdS, &CMapS, &ImgS, ulx, uly, lrx, lry, ALLOC ) ) 
      exit(0);
   fclose( fpis );

   /* remap the images */
   fprintf( stderr,"remapping images\n");
   MapThroughMap( &RasHdL, CMapL, ImgL, ulx, uly, lrx, lry );
   MapThroughMap( &RasHdH, CMapH, ImgH, ulx, uly, lrx, lry );
   MapThroughMap( &RasHdS, CMapS, ImgS, ulx, uly, lrx, lry );
   if ( CMapL ) free( CMapL );
   if ( CMapH ) free( CMapH );
   if ( CMapS ) free( CMapS );

   /* get the size */
   fprintf(stderr,"geting image size \n");
   X = RasHdH.ras_width ;
   Y = RasHdH.ras_height;
   N = X*Y;

   /* check that the images are of the same size */
   fprintf(stderr,"comparing image sizes \n");
   if ( (RasHdH.ras_width != X)  ||  (RasHdH.ras_height != Y) || 
        (RasHdS.ras_width != X)  ||  (RasHdS.ras_height != Y) )
      {
      fprintf(stderr, "The three input images have unequal dimensions\n");
      exit( 0 );
      }

   /* allocate output images */
   fprintf(stderr,"allocating output images \n");
   if ( !(DilL = (byte *)calloc( ((X+sX-1)*(Y+sY-1)), sizeof(byte) )) )
         {
         fprintf(stderr,"Unable to allocate space for lum output image\n");
         exit( 0 );
         } 
   if ( !(DilH = (byte *)calloc( ((X+sX-1)*(Y+sY-1)), sizeof(byte) )) )
         {
         fprintf(stderr,"Unable to allocate space for hue output image\n");
         exit( 0 );
         } 
   if ( !(DilS = (byte *)calloc( ((X+sX-1)*(Y+sY-1)), sizeof(byte) )) )
         {
         fprintf(stderr,"Unable to allocate space for sat output image\n");
         exit( 0 );
         } 

   /* Do dilation */
   fprintf(stderr,"performing dilation \n");
   if ( optype == LUM )
      LumSetDilate( ImgL, ImgH, ImgS, DilL, DilH, DilS, X+sX-1, X, Y, SE, sX, sY, sorgx, sorgy );
   else if ( optype == SAT )
      SatSetDilate( ImgL, ImgH, ImgS, DilL, DilH, DilS, X+sX-1, X, Y, SE, sX, sY, sorgx, sorgy );
   else
      HueSetDilate( ImgL, ImgH, ImgS, DilL, DilH, DilS, X+sX-1, X, Y, SE, sX, sY, sorgx, sorgy );

   /* create a grayscale colormap */
   fprintf(stderr,"making grayscale colormap\n");
   MapGS = MakeColorMap();                        

   /* open output files */
   fprintf(stderr,"opening output files\n");
   if ( !( fpol = fopen( OutNameLum, "w" )) )
      {
      fprintf( stderr, "Unable to open %s\n", OutNameLum );
      exit( 0 );
      }
   if ( !( fpoh = fopen( OutNameHue, "w" )) )
      {
      fprintf( stderr, "Unable to open %s\n", OutNameHue );
      exit( 0 );
      }
   if ( !( fpos = fopen( OutNameSat, "w" )) )
      {
      fprintf( stderr, "Unable to open %s\n", OutNameSat );
      exit( 0 );
      }

   /* write out files */
   fprintf(stderr,"writing output files\n");
   WriteRasterFile( fpol, &RasHdL, MapGS, DilL, ulx, uly, lrx, lry );
   fclose( fpol );
   WriteRasterFile( fpoh, &RasHdH, MapGS, DilH, ulx, uly, lrx, lry );
   fclose( fpoh );
   WriteRasterFile( fpos, &RasHdS, MapGS, DilS, ulx, uly, lrx, lry );
   fclose( fpos );

   free ( MapGS );
   free ( ImgL );
   free ( ImgH );
   free ( ImgS );
   free ( DilL );
   free ( DilH );
   free ( DilS );
   free ( SE );
   fprintf(stderr,"done\n");

   }


/* Get the structuring element from a file. Allocate the space */
/* for it and return a pointer to it.                          */

int *LoadSE( sname, sX, sY, sorgx, sorgy )
   char *sname;        /* structuring element file name */
   int *sX,*sY;        /* x,y dimensions of bounding box */
   int *sorgx, *sorgy; /* x,y position of SE origin */
   {    
   int i,n;            /* indicies */
   int *SE;            /* SE array */
   char *prefix;       /* leading component of pathnmae */
   char pathname[256]; /* pathname string */
   FILE *sefp;         /* structuring element file pointer */

   /* if environment variable SEPATH is set, get it and prepend to path */
   pathname[0] = '\0';
   prefix = getenv( SEPATH );
   if ( prefix && (n = strlen(prefix))  ) 
      {
      strcpy(pathname,prefix);
      if ( pathname[n-1] != '/' )
         {
         pathname[n] = '/';
         pathname[n+1] = '\0';
         }
      }
   strcat( pathname, sname );

   /* try to open the SE file */
   if ( !(sefp = fopen( pathname, "r")) )
      {
      fprintf(stderr,"Unable to open %s\n", pathname);
      exit(0);
      }

   /* get the xy dimensions and origin coordinates */
   fscanf(sefp,"%d %d %d %d",sX,sY,sorgx,sorgy);

   /* make sure origin is within bounding box */
   if ( *sorgx>=*sX || *sorgy>=*sY )
      {
      fprintf(stderr,"The SE origin must be inside the bounding rectangle.\n");
      exit(0);
      }

   /* number of pixels covered by bounding box */
   n = (*sX) * (*sY);

   /* allocate the space for it */
   if ( !(SE = (int *)calloc( n, sizeof(int) )) )
      {
      fprintf(stderr,"Unable to allocate memory for SE %s\n", sname);
      exit(0);
      }

   /* read it in from the file */
   for ( i=0; i<n; ++i ) fscanf(sefp, "%d", SE+i);

   /* close the file */
   fclose( sefp );

   /* pass the pointer to the SE array */
   return( SE );
   }

void MapThroughMap( RasHd, CMap, Img, ulx, uly, lrx, lry )
   struct rasterfile *RasHd;	/* raster file header */
   byte *CMap;
   byte *Img;
   int ulx, uly, lrx, lry;
   {
   byte *I;
   byte *R, *G, *B;
   int X,Y,H,W;
   int i,j;

   if ( CMap == (byte *)NULL ) return;
   
   R = CMap;
   G = R + RasHd->ras_maplength/3;
   B = G + RasHd->ras_maplength/3;
   
   X = RasHd->ras_width;
   Y = RasHd->ras_height;
   W = X + ulx + lrx;
   H = Y + uly;

   fprintf(stderr,"MapThroughMap: X=%d Y=%d W=%d H=%d\n",X,Y,W,H);
   for ( j=uly; j<H; ++j );
      {
      I = Img + j*W + ulx;
      for ( i=0; i<X; ++i );
         {
         *I = (*(R+*I) + *(G+*I) + *(B+*I)) /3;
         ++I;
         }
      }
   fprintf(stderr,"Done remapping\n");
   
   return;
   }
   

void LumSetDilate( InL, InH, InS, OutL, OutH, OutS, X, tX, tY, stelem, sX, sY, sorgx, sorgy ) 
   byte *InL,*InH,*InS;    /* input luminance, hue and saturation images */
   byte *OutL,*OutH,*OutS; /* output luminance, hue and saturation images */
   int X;            /* image horizontal dimension */
   int tX,tY;        /* transform scan horiz., vert. dimensions */
   int *stelem;      /* SE in row-major order */
   int sX,sY;        /* SE horizontal, vertical dimensions */
   int sorgx,sorgy;  /* SE origin offset from upper LH corner. */
   {

   byte *IL,*JL,*KL,*LL;      /* luminance image pointers */
   byte *OL,*PL;              /* luminance image pointers */
   byte *IH,*JH,*KH,*LH;      /* hue image pointers */
   byte *OH,*PH;              /* hue image pointers */
   byte *IS,*JS,*KS,*LS;      /* saturation image pointers */
   byte *OS,*PS;              /* saturation image pointers */
   int i,j;                   /* transform indices */
   int x,y;                   /* image indices     */
   int *s;                    /* pointer to SE     */
   int lum,hue,sat,hdiff;            /* lum, hue, sat, & hue diff w/ SE at pixel */
   int maxlum,maxhue,maxsat,maxdiff; /* maximum lum, hue, sat and diff in nbhd. */

   IL = InL;                    /* first lum input pixel  */
   OL = OutL + (sY-sorgy-1)*X + sX-sorgx-1; /* first lum output pixel */
   IH = InH;                    /* first hue input pixel  */
   OH = OutH + (sY-sorgy-1)*X + sX-sorgx-1; /* first hue output pixel */
   IS = InS;                    /* first sat input pixel  */
   OS = OutS + (sY-sorgy-1)*X + sX-sorgx-1; /* first sat output pixel */

   for ( y=0; y<tY; ++y )      /* image row scan */ 
      {
      JL = IL; /* first lum pixel in nbhd index */
      PL = OL; /* first lum output pixel in row index */
      JH = IH; /* first hue pixel in nbhd index */
      PH = OH; /* first hue output pixel in row index */
      JS = IS; /* first sat pixel in nbhd index */
      PS = OS; /* first sat output pixel in row index */

      for ( x=0; x<tX; ++x )   /* image column scan */
         {
         s  = stelem + sX*sY - 1; /* first element in SE */ 
         KL = JL++;  /* first lum pixel in nbhd row index */
         KH = JH++;  /* first hue pixel in nbhd row index */
         KS = JS++;  /* first sat pixel in nbhd row index */
         maxlum  = ZERO;   /* result MIN at first */
         maxhue  = ZERO;   /* result MIN at first */
         maxsat  = ZERO;   /* result MIN at first */
         maxdiff = ZERO;   /* result MIN at first */

         for ( j=0; j<sY; ++j )     /* SE row scan */
            {
            LL = KL;  /* lum input image neighborhood pixel index */
            LH = KH;  /* hue input image neighborhood pixel index */
            LS = KS;  /* sat input image neighborhood pixel index */

            for ( i=0; i<sX; ++i )  /* SE column scan */
               {
               if (*s >= ZERO)  
                  {
                  lum   = *(LL++);      /* current lum */
                  hue   = *(LH++);      /* current hue */
                  sat   = *(LS++);      /* current saturation */
                  hdiff = hue - *(s--); /* Hue difference with SE value */
                  if ( hdiff > HPI )      /* CW is closer */
                     hdiff -= H2PI;
                  else if ( hdiff < -HPI) /* CCW is closer */
                     hdiff += H2PI;
                  if ( lum > maxlum ) 
                     {               
                     maxdiff = hdiff;
                     maxlum  = lum;
                     maxhue  = hue;
                     maxsat  = sat;
                     }
                  else if ( lum == maxlum )
                     {
                     if ( abs(hdiff) > abs(maxdiff) )  
                        {
                        maxdiff = hdiff;
                        maxlum  = lum;
                        maxhue  = hue;
                        maxsat  = sat;  
                        }
                     else if ( (hdiff == maxdiff )  && (sat < maxsat) );
                        {                        /* if hue diffs are same, sats are the same */
                        maxlum  = lum;           /* but lum is larger, take the current      */
                        maxhue  = hue;           /* lum, hue, and sat values.                */
                        maxsat  = sat;           
                        }
                      }
                  else
                     { }
                  }
               else /* inactive SE element */
                  { ++LL; ++LH; ++LS; --s; }
               }                    /* end of SE column scan */ 

            KL += X;  /* first pixel in next row of nbhd */
            KH += X;  /* first pixel in next row of nbhd */
            KS += X;  /* first pixel in next row of nbhd */
            }                       /* end of SE row scan */

         *(PL++) = maxlum; /* output lum result */
         *(PH++) = maxhue; /* output hue result */
         *(PS++) = maxsat; /* output sat result */
         }                          /* end of image column scan */

      IL += X; /* first input pixel in next row  */
      OL += X; /* first output pixel in next row */
      IH += X; /* first input pixel in next row  */
      OH += X; /* first output pixel in next row */
      IS += X; /* first input pixel in next row  */
      OS += X; /* first output pixel in next row */
      }                             /* end of image row scan */
   return;
   }                                /* end of LumSetDilate */





void HueSetDilate( InL, InH, InS, OutL, OutH, OutS, X, tX, tY, stelem, sX, sY, sorgx, sorgy ) 
   byte *InL,*InH,*InS;    /* input luminance, hue and saturation images */
   byte *OutL,*OutH, *OutS;  /* output hue and saturation images */
   int X;            /* image horizontal dimension */
   int tX,tY;        /* transform scan horiz., vert. dimensions */
   int *stelem;      /* SE in row-major order */
   int sX,sY;        /* SE horizontal, vertical dimensions */
   int sorgx,sorgy;  /* SE origin offset from upper LH corner. */
   {

   byte *IL,*JL,*KL,*LL;      /* luminance image pointers */
   byte *OL,*PL;              /* luminance image pointers */
   byte *IH,*JH,*KH,*LH;      /* hue image pointers */
   byte *OH,*PH;              /* hue image pointers */
   byte *IS,*JS,*KS,*LS;      /* saturation image pointers */
   byte *OS,*PS;              /* saturation image pointers */
   int i,j;                   /* transform indices */
   int x,y;                   /* image indices     */
   int *s;                    /* pointer to SE     */
   int lum,hue,sat,hdiff;            /* lum, hue, sat, & hue diff w/ SE at pixel */
   int maxlum,maxhue,maxsat,maxdiff; /* maximum lum, hue, sat and diff in nbhd. */

   IL = InL;                    /* first lum input pixel  */
   OL = OutL + (sY-sorgy-1)*X + sX-sorgx-1; /* first lum output pixel */
   IH = InH;                    /* first hue input pixel  */
   OH = OutH + (sY-sorgy-1)*X + sX-sorgx-1; /* first hue output pixel */
   IS = InS;                    /* first sat input pixel  */
   OS = OutS + (sY-sorgy-1)*X + sX-sorgx-1; /* first sat output pixel */

   for ( y=0; y<tY; ++y )      /* image row scan */ 
      {
      JL = IL; /* first lum pixel in nbhd index */
      PL = OL; /* first lum output pixel in row index */
      JH = IH; /* first hue pixel in nbhd index */
      PH = OH; /* first hue output pixel in row index */
      JS = IS; /* first sat pixel in nbhd index */
      PS = OS; /* first sat output pixel in row index */

      for ( x=0; x<tX; ++x )   /* image column scan */
         {
         s  = stelem + sX*sY - 1; /* first element in SE */ 
         KL = JL++;  /* first lum pixel in nbhd row index */
         KH = JH++;  /* first hue pixel in nbhd row index */
         KS = JS++;  /* first sat pixel in nbhd row index */
         maxlum  = ZERO;   /* result MIN at first */
         maxhue  = ZERO;   /* result MIN at first */
         maxsat  = ZERO;   /* result MIN at first */
         maxdiff = ZERO;   /* result MIN at first */

         for ( j=0; j<sY; ++j )     /* SE row scan */
            {
            LL = KL;  /* lum input image neighborhood pixel index */
            LH = KH;  /* hue input image neighborhood pixel index */
            LS = KS;  /* sat input image neighborhood pixel index */

            for ( i=0; i<sX; ++i )  /* SE column scan */
               {
               if (*s >= ZERO)  
                  {
                  lum   = *(LL++);      /* current lum */
                  hue   = *(LH++);      /* current hue */
                  sat   = *(LS++);      /* current saturation */
                  hdiff = hue - *(s--); /* Hue difference with SE value */
                  if ( hdiff > HPI )      /* CW is closer */
                     hdiff -= H2PI;
                  else if ( hdiff < -HPI) /* CCW is closer */
                     hdiff += H2PI;
                  if ( abs(hdiff) > abs(maxdiff) ) /* larger of  */
                     {                             /* two angles */
                     maxdiff = hdiff;
                     maxlum  = lum;
                     maxhue  = hue;
                     maxsat  = sat;
                     }
                  else if ( hdiff == maxdiff )
                     {
                     if (sat > maxsat )  
                        {                /* if hue diffs are same, but the current  */
                        maxlum  = lum;   /* saturation is larger, take the current  */
                        maxhue  = hue;   /* lum, hue, and sat values.               */
                        maxsat  = sat;  
                        }
                     else if ( (sat == maxsat )  && (lum > maxlum) );
                        {                        /* if hue diffs are same, sats are the same */
                        maxlum  = lum;           /* but lum is larger, take the current      */
                        maxhue  = hue;           /* lum, hue, and sat values.                */
                        maxsat  = sat;           
                        }
                      }
                  else
                     { }
                  }
               else /* inactive SE element */
                  { ++LL; ++LH; ++LS; --s; }
               }                    /* end of SE column scan */ 

            KL += X;  /* first pixel in next row of nbhd */
            KH += X;  /* first pixel in next row of nbhd */
            KS += X;  /* first pixel in next row of nbhd */
            }                       /* end of SE row scan */

         *(PL++) = maxlum; /* output lum result */
         *(PH++) = maxhue; /* output hue result */
         *(PS++) = maxsat; /* output sat result */
         }                          /* end of image column scan */

      IL += X; /* first input pixel in next row  */
      OL += X; /* first output pixel in next row */
      IH += X; /* first input pixel in next row  */
      OH += X; /* first output pixel in next row */
      IS += X; /* first input pixel in next row  */
      OS += X; /* first output pixel in next row */
      }                             /* end of image row scan */
   return;
   }                                /* end of HueSetDilate */


void SatSetDilate( InL, InH, InS, OutL, OutH, OutS, X, tX, tY, stelem, sX, sY, sorgx, sorgy ) 
   byte *InL,*InH,*InS;    /* input luminance, hue and saturation images */
   byte *OutL,*OutH,*OutS; /* output luminance, hue and saturation images */
   int X;            /* image horizontal dimension */
   int tX,tY;        /* transform scan horiz., vert. dimensions */
   int *stelem;      /* SE in row-major order */
   int sX,sY;        /* SE horizontal, vertical dimensions */
   int sorgx,sorgy;  /* SE origin offset from upper LH corner. */
   {
   fprintf(stderr,"sat not yet implemented\n");
   exit(0);
   return;
   }







 
