/* testmorph.c  --  tester for morph */

/* contains: main() */

/* dependencies: GetArgs(), ReadRasterFile(), MorphSub(), WriteRasterFile() */
/* ExtractLuminance()

/* morph version 4.0  10 May 1993                  */    
/* image morphology program                        */
/*                                                 */
/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */
/* Nashville, TN 37235                             */
/* rap2@vuse.vanderbilt.edu                        */
/*                                                 */
/* This software is freely redistributable if      */
/* the author's name and affiliation are included. */

/* See changes.doc for an explanation of the changes in v4.0. */

#include <string.h>
#include "matrix.h"
#include "morph_sub.h"

#define MAX_STRING 256
#define MORPH_DILATE 2   /* morphological dilation operator */
#define MORPH_RANK 16   /* morphological rank operator */
#define MORPH_GRAIMG 0 /* values in Img will be gray level */
#define MORPH_BINIMG 512 /* values in Img will be 0 and non-zero */
#define MORPH_GRASE 0 /* values in Structuring element will be 0 and non-zero */
#define MORPH_BINSE 256 /* values in Structuring element will be 0 and non-zero */
#define MORPH_NZPAD_OFF 0    /* padding off */
#define MORPH_THRESH_OFF 0   /* thresholding off */
#define MORPH_SQUARE_SE 3 /* square 3x3 structuring element */
#define MORPH_SET 0  /* SorF set operation */
#define MORPH_NOSCALE 0  /* dummy for binary data */
#define MORPH_VERBOSE 1 /* turnon verbosity */

void printImg( byte *Img, int width, int height);

main( argc, argv ) 
   unsigned int argc;  /* count of arguments parsed by operating system    */
   char *argv[];       /* pointers to arguments parsed by operating system */
   {

   unsigned char *cmap;      /* colormap */

   int MorphOp;   /* operation to perform */

   byte **ImgMatrix;     /* image as list in row-major order */
   byte **OutImgMatrix;     /* image as list in row-major order */
   byte *Img;     /* image as list in row-major order */
   byte *OutImg;     /* image as list in row-major order */
   int X,Y;       /* image horizontal, vertical dimensions */
   int ImgType;   /* gray-level or binary */
   int NZpad ;    /* flag.  F => zeropadding of input */
   int NoScale;   /* flag.  T => do not scale output of IntMorph */
   int Dsp;       /* flag.  T => display some info */

   char *SEname;  /* structuring element (SE) path name */
   char test_string[MAX_STRING];  
   int sX,sY;     /* SE horizontal, vertical dimensions */
   int sZ;        /* largest gray level in SE */
   int autoSE;    /* T => create a disk-shaped SE locally */
   int SEType;    /* binary or gray-level SE */
   int SorF;      /* Set operation or Function operation */

   int LThresh;   /* lower binary threshold value */
   int UThresh;   /* upper binary threshold value */
   int rank;      /* rank for rank filter */
   


   /* begin */
   strncpy( test_string, "/this/is/a/test", MAX_STRING );
   fprintf( stderr, "Test string is:%s\n", test_string );


   /* define a test image */
   X = 15;
   Y = 10;
/*   Img = (byte *)malloc( sizeof(byte) * X * Y);
     OutImg = (byte *)malloc( sizeof(byte) * X * Y);*/
   ImgMatrix = (byte **)matrix( X, Y, sizeof(byte), matrix_ZERO );
   OutImgMatrix = (byte **)matrix( X, Y, sizeof(byte), matrix_ZERO );
   Img = *ImgMatrix;
   OutImg = *OutImgMatrix;
   Img[11] = 1;
   Img[44] = 1;
   Img[45] = 1;
   Img[46] = 1;
   Img[81] = 1;
   Img[91] = 1;
   fprintf( stderr, "Starting Img array:\n");
   printImg( Img, X, Y );

   /* Set input options */
/*   SEname = strdup("/home/brodzik/branches/pmalgos/morph/kernels/nsidc_roundSE3x3.txt");*/
   SEname = strdup("/home/brodzik/branches/pmalgos/morph/kernels/nsidc_roundSE3x3.txt");
   autoSE = 0;
   

   /* do the morph op */
   MorphSub( MORPH_DILATE, Img, OutImg, X, Y, MORPH_BINIMG, MORPH_NZPAD_OFF,
             1, 1, SEname, autoSE,
             sX, sY, sZ, MORPH_BINSE, MORPH_SET, 0, 0, MORPH_VERBOSE);

   /* write the result */
   fprintf( stderr, "\n" );
   fprintf( stderr, "Ending Img array:\n");
   printImg( Img, X, Y );
   fprintf( stderr, "\n" );
   fprintf( stderr, "Ending OutImg array:\n");
   printImg( OutImg, X, Y );

   free( ImgMatrix );
   free( OutImgMatrix );

   /* test the rank operator to see if it does what I think */
   /* define a test image */
   X = 10;
   Y = 10;
   ImgMatrix = (byte **)matrix( X, Y, sizeof(byte), matrix_ZERO );
   OutImgMatrix = (byte **)matrix( X, Y, sizeof(byte), matrix_ZERO );
   ImgMatrix[1][1] = 1;
   ImgMatrix[4][4] = 1;
   ImgMatrix[4][5] = 1;
   ImgMatrix[4][6] = 1;
   ImgMatrix[8][1] = 1;
   ImgMatrix[7][7] = 1;
   ImgMatrix[7][8] = 1;
   ImgMatrix[8][7] = 1;
   ImgMatrix[9][1] = 1;
   
   fprintf( stderr, "Starting Img array:\n");
   printImg( *ImgMatrix, X, Y );

   /* Set input options */
   SEname = strdup("/share/data/pmalgos/spillover_kernels/square3x3.txt");
   autoSE = 0;

   /* do a rank filter:  find all locations with at least rank k active elements
    * in the neighborhood defined by the kernel */
   rank = 2;
   MorphSub( MORPH_RANK, *ImgMatrix, *OutImgMatrix, X, Y, MORPH_BINIMG, MORPH_NZPAD_OFF,
             1, 1, SEname, autoSE,
             sX, sY, sZ, MORPH_BINSE, MORPH_SET, rank, 0, MORPH_VERBOSE);

   /* write the result */
   fprintf( stderr, "\n" );
   fprintf( stderr, "Ending ImgMatrix array:\n");
   printImg( *ImgMatrix, X, Y );
   fprintf( stderr, "Ending OutImgMatrix array:\n");
   printImg( *OutImgMatrix, X, Y );

   free( ImgMatrix );

   exit( 0 );
   
   }


/* prints the test array to stderr */
void printImg( byte *Img, int width, int height) {

    int col, row;

    fprintf( stderr, "Begin Img array:\n" );
    for ( row = 0; row < height; row++ ) {
        for ( col = 0; col < width; col++ ) {
            fprintf( stderr, "%3d ", *(Img + ( row * width ) + col ) );
        }
        fprintf( stderr, "\n" );
    }
    fprintf( stderr, "End Img array\n" );

    return;
    
}
