/* openspec - compute the opening spectrum of an image  */

/* dependencies: ReadRasterFile(), MorphSub(), WriteRasterFile()   */

/* openspec version 1.0   8 May 1994             */
/*                                                 */
/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */
/* Nashville, TN 37235                             */
/* rap2@vuse.vanderbilt.edu                        */
/*                                                 */
/* This software is freely redistributable if      */
/* the author's name and affiliation are included. */

#include "morph_sub.h"
#include "rasterio.h"
#include "openspec.h"

void DumpFile();
int  ImgArea();

main( argc, argv ) 
   unsigned int argc;    /* count of arguments parsed by operating system    */
   char *argv[];         /* pointers to arguments parsed by operating system */
   {
   struct rasterfile H;    /* raster file header */
   unsigned char *C;       /* colormap */
   byte  *I,*O,*T;         /* original image, open image, and temp image */
   FILE  *fp;              /* file pointer */
   int    SEdim[NUMBANDS]; /* dimensions of SEs (not file based) */
   int    SEtype;          /* disk, horiz. line, vert. line, def'd. in file */
   int    order;           /* number of openings to perform */
   int    i,j,k;
   int    M,X,Y;
   int   *Spec,*S;
   int    verbose;
   char   fname[256];
   char   *dot;

   /* begin */



   if (argc < 4 ) /* display usage */
      {
      fprintf(stderr,
         "usage: patternspec  InFile  OutFileBaseName  order\n");
      exit(0);
      }


   /* Get the arguments */

   order = atoi(argv[3]);
   

   /* make the numerical array */
   if ( !( Spec = (int *)calloc( order*order, sizeof(int) )) )
      {
      fprintf(stderr,"main: Unable to allocate pattern spec buffer.\n");
      exit(0);
      } 


   /* read the input image into memory */

   if ( !(fp = OpenFile( argv[1], "IMGPATHI", "r" )) ) exit( 0 );
   ReadRasterFile( fp, &H, &C, &I, 0, 0, 0, 0, ALLOC );
   fclose( fp );


   /* extract the luminance */

   ExtractLuminance( &H, C, I, 0, 0, 0, 0 );


   /* get the image dimensions */

   X = H.ras_width;
   Y = H.ras_height;
   M = X*Y;


   /* allocate image space */

   if ( !( O = (byte *)calloc( M, sizeof(byte) )) )
      {
      fprintf(stderr,"main: Unable to allocate image buffer.\n");
      exit(0);
      } 


   /* set up output filename */
   strcpy( fname, argv[2] );
   dot = rindex( fname, '.' );
   if ( !dot || (dot == fname) ) 
      dot = fname + strlen( fname );

   /* vertical then horizontal */

   /* first row */
   *Spec = ImgArea( I, X, Y );
   for ( i=1; i<order; ++i )
      {
      fprintf( stderr, " 1 %d\n", i);
      MorphSub( OPEN,I,O,X,Y,GRAIMG,FALSE,0,0,NULL,AUTO,
                (i+i),1,0,GRASE,SET,0,FALSE,FALSE);
      *(Spec+i) = ImgArea( O, X, Y );
      }

   /* other rows */
   for ( j=1; j<order; ++j )
      {
      MorphSub( OPEN,I,O,X,Y,GRAIMG,FALSE,0,0,NULL,AUTO,
                1,(j+1),0,GRASE,SET,0,FALSE,FALSE);
      *(Spec+(j*order)) = ImgArea( O, X, Y );

      for ( i=1; i<order; ++i )
         {
         MorphSub( OPEN,O,O,X,Y,GRAIMG,FALSE,0,0,NULL,AUTO,
                   (i+i),1,0,GRASE,SET,0,FALSE,FALSE);
         *(Spec+(j*order)+i) = ImgArea( O, X, Y );
         }
      }

   DumpFile( fname, dot, ".1", Spec, order );


   /* horizontal then vertical */

   /* first column  */

   for ( j=1; j<order; ++j )
      {
      MorphSub( OPEN,I,O,X,Y,GRAIMG,FALSE,0,0,NULL,AUTO,
                1,(j+i),0,GRASE,SET,0,FALSE,FALSE);
      *(Spec+(j*order)) = ImgArea( O, X, Y );
      }


   /* other columns */

   for ( i=1; i<order; ++i )
      {
      MorphSub( OPEN,I,O,X,Y,GRAIMG,FALSE,0,0,NULL,AUTO,
                (i+1),1,0,GRASE,SET,0,FALSE,FALSE);
      *(Spec+i) = ImgArea( O, X, Y );

      for ( j=1; j<order; ++j )
         {
         MorphSub( OPEN,O,O,X,Y,GRAIMG,FALSE,0,0,NULL,AUTO,
                   1,(j+i),0,GRASE,SET,0,FALSE,FALSE);
         *(Spec+(j*order)+i) = ImgArea( O, X, Y );
         }
      }

   DumpFile( fname, dot, ".2", Spec, order );

   free( I );
   free( O );
   if ( C ) free( C );

   }



static char *rindex( s, c)
   char *s;
   char c;
   {
   int n;
   char *t;

   n = strlen( s );
   t = s+n-1;
   while ( n && (*t != c) )
      {
      --t;
      --n;
      }
   if ( n )
      return( t );
   else
      return( NULL );
   }


     
int ImgArea( Img, X, Y )
   byte *Img;
   int  X,Y;
   {
   byte *I;
   int acc;
   int i,N;

   N = X*Y;
   I = Img;
   for ( i=N; i; --i)
      acc = *(I++);
   acc /= WHITE;

   return( acc );
   }

   

void DumpFile( fname, dot, tag, Spec, order )
   char *fname;
   char *dot;
   char *tag;
   int  *Spec;
   int  order;
   {
   int i,j;
   FILE  *fp;              /* file pointer */

   sprintf( dot, tag );
   strcat( fname, ".ras" );

   if ( !(fp = OpenFile( fname, NULL, "w" )) ) exit( 0 );

   for ( j=0; j<order; ++j )
      {
      for ( i=0; i<order; ++i )
         {
         fprintf( fp, " %6d", *(Spec+(j*order)+i) );
         }
      fprintf( fp, "\n" );
      }

   fclose( fp );
   }


