/* thsmooth2.c  --  thsmooth2 vers. 1.0 main program */

/* dependencies: ReadRasterFile(), MorphSub(), WriteRasterFile()   */

/* thsmooth2 version 1.0   5 July 1994                */
/*                                                 */
/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */
/* Nashville, TN 37235                             */
/* rap2@vuse.vanderbilt.edu                        */
/*                                                 */
/* This software is freely redistributable if      */
/* the author's name and affiliation are included. */


#include "thsmooth2.h"
#include "rasterio.h"
#include "morph_sub.h"

static   struct rasterfile RasHd;    /* raster file header */
static   unsigned char *cmap;        /* colormap */
 
main( argc, argv ) 
   unsigned int argc;    /* count of arguments parsed by operating system    */
   char *argv[];         /* pointers to arguments parsed by operating system */
   {

   byte *Img[NBANDS];      /* input images */
   byte *Out;              /* output image */
   byte *TH,*VR;           /* thresholded image, reconstructed image  */
   byte *Com;              /* Combined images */
   byte *I,*O,*R,*T;       /* image pointers */
   int X,Y;                /* image horiz, vert. dimensions  */
   int N,M;                /* number of pixels in image, number of images */
   int t,r;                /* threshold, residual threshold */
   double m,tmf;           /* slope, threshold multiplication factor */
   int i,n;                /* index variables    */
   FILE *fp;               /* file pointer  */
   char bname[FNLGTH];     /* name of input file basename */
   char iname[FNLGTH];     /* name of input file */
   char fname[FNLGTH];     /* name of output file */
   char *dot;              /* pointer to '.' in filename string */



   /* begin */



   if (argc < 4 ) /* display usage */
      {
      fprintf(stderr,
         "usage:  thsmooth2 InBaseName OutFile bands [-f nn.nn] \n");
      exit(0);
      }


   /* Get the arguments */
   GetCLArgs(argc,argv,bname,fname,&M,&tmf);


   /* read the input images into memory */
   for( i=0; i<M; ++i )
      {
      strcpy( iname, bname );
      dot = iname + strlen( iname );
      sprintf( dot, ".%03d", i );
      strcat( iname, ".ras" );
      ReadIt( iname, &RasHd, &cmap, &(Img[i]), ALLOC );
      }

   /* get the image dimensions */

   X = RasHd.ras_width;
   Y = RasHd.ras_height;
   N = X*Y;


   /* allocate output and intermediate images */

   if ( !( Out = (byte *)calloc( N, sizeof(byte) )) ) /* output image */
      {
      fprintf(stderr,"main: Unable to allocate output image.\n");
      exit(0);
      }

   if ( !( TH = (byte *)calloc( N, sizeof(byte) )) ) /* thresholded image */
      {
      fprintf(stderr,"main: Unable to allocate thresholded image.\n");
      exit(0);
      }

   if ( !( VR = (byte *)calloc( N, sizeof(byte) )) ) /* reconstructed image */
      {
      fprintf(stderr,"main: Unable to allocate reconstructed image.\n");
      exit(0);
      }

   /* first image */
   t = Moment2( Img[0], N );
   t = (int)(tmf*(double)t + 0.5);
   Thresh( Img[0], TH, N, t );
   ReconSub( Img[0], TH, VR, X, Y, 4, FALSE );
   memcpy( VR, Out, N );

/***WriteIt( "thresh.ras", &RasHd, cmap, TH );*/

   /* main algorithm loop */

   for ( i=1; i<M-1; ++i )
      {
      I = Img[i];
      t = Moment2( I, N );
      t = (int)(tmf*(double)t + 0.5);
      ThreshRef( I, Out, TH, N, t, 1);
      ReconSub( I, TH, VR, X, Y, 4, FALSE );

/***WriteIt( "thresh.ras", &RasHd, cmap, TH );*/

      AddImg( VR, Out, Out, X, Y );

/***WriteIt( "out.ras", &RasHd, cmap, Out );*/

       }

   AddImg( Img[M-1], Out, Out, X, Y );

/***WriteIt( "out.ras", &RasHd, cmap, Out );*/

   /* output cleaned image */

   WriteIt( argv[2], &RasHd, cmap, Out );



   /* free memory */

   for ( i=1; i<M-1; ++i )
      {
      free( Img[i] );
      }
   free( Out );
   free( TH );
   free( VR );
   if (cmap) free(cmap);

   } /* end */




static void AddImg( I1, I2, O, X, Y )
   byte *I1;  /* image 1 */
   byte *I2;  /* image 2 */
   byte *O;   /* result  */
   int X,Y;   /* image horizontal, vertical dimensions */
   {
   byte *A = I1;
   byte *B = I2;
   byte *C = O;
   int   n = X*Y;
   int i,p;

   for (i=0; i<n; ++i)
      {
      p = (int)(*(A++)) + (int)(*(B++));
      *(C++) = (p > WHITE) ? WHITE : ( (p < BLACK) ? BLACK : p );
      }

   return;
   }



static void AndImg ( I1, I2, O, X, Y )
   byte *I1;  /* image 1 */
   byte *I2;  /* image 2 */
   byte *O;   /* result  */
   int X,Y;   /* image horizontal, vertical dimensions */
   {
   byte *A = I1;
   byte *B = I2;
   byte *C = O;
   int   n = X*Y;
   int i;

   for (i=0; i<n; ++i)
      {
      *C = (*A && *B) ? *A : BLACK;
      ++A;
      ++B;
      ++C;
      }

   return;
   }

      

static int Moment2( Img, N )
   byte *Img;
   int   N;
   {
   byte *I;
   int i;
   int histo[WHITE+1];
   int *h;
   int area;
   int t;
   double a,m,p;

   /* calculate threshold as the 2nd moment */
   /* of the histogram othe residual image. */

   /* clear histogram */
   memset( histo, '\000', sizeof(int)*(WHITE+1) );

   /* collect histogram */
   I = Img;
   for (i=0; i<N; ++i)  
      (*(histo + *(I++)))++;    

   /* calculate area of histogram */
   area = 0;
   h = histo;
   for (i=0; i<=WHITE; ++i)  
      area += *(h++);

   /* calculate second moment of histogram */
   a = (double)area;
   h = histo; 
   m = 0;
   for (i=0; i<=WHITE; ++i)
      {
      p  = (double)(*(h++)) / a;
      m += p * (double)(i*i);
      }
   m = sqrt(m);
   t = (int)(m + 0.5);

   return(t);
   }



static void Thresh( Img, Thr, N, t )
   byte *Img,*Thr;
   int   N;
   int   t;
   {
   byte *I,*T;
   int i;


   /* threshold image */
   I = Img;
   T = Thr;
   for ( i=N; i; --i )
      {
      if (*I >= t)
         *(T++) =  *I;
      else
         *(T++) = 0;
      ++I;
      }

   return;
   }



static void ThreshRef( Img, Ref, Thr, N, t, r )
   byte *Img,*Ref,*Thr;
   int   N;
   int   t,r;
   {
   byte *I,*R,*T;
   int i;


   /* threshold image */
   I = Img;
   R = Ref;
   T = Thr;
   for ( i=N; i; --i )
      {
      if ( (*I >= t) || (*R >= r) )
         *(T++) =  *I;
      else
         *(T++) = 0;
      ++I;
      ++R;
      }

   return;
   }



static void ReadIt( name, H, C, I, Alloc )
   char *name;                /* filename */
   struct rasterfile *H;      /* raster file header */
   unsigned char **C;         /* colormap */
   byte **I;                  /* image */
   int Alloc;                 /* allocate flag */
   {
   FILE *fp;

   if ( !(fp = OpenFile( name, "IMGPATHI", "r" )) ) exit( 0 );
   ReadRasterFile( fp, H, C, I, 0, 0, 0, 0, Alloc );
   fclose( fp );

   /* extract the luminance */
   ExtractLuminance( H, *C, *I, 0, 0, 0, 0 );

   return;
   }


static void WriteIt( name, H, C, I )
   char *name;                /* filename */
   struct rasterfile H;       /* raster file header */
   unsigned char *C;          /* colormap */
   byte *I;                   /* image */
   {
   FILE *fp;

   if ( !(fp = OpenFile( name, "IMGPATHO", "w" )) ) exit( 0 );
   WriteRasterFile( fp, H, C, I, 0, 0, 0, 0 );
   fflush( fp );
   fclose( fp );

   return;
   }




static void OptErr( s, t )
   char *s,*t;
   {
   fprintf(stderr,"Invalid Op: %s %s\n",s,t);
   exit( 0 );
   }




static char *rindex( s, c)
   char *s;
   char c;
   {
   int n;
   char *t;

   n = strlen( s );
   t = s+n-1;
   while ( n && (*t != c) )
      {
      --t;
      --n;
      }
   if ( n )
      return( t );
   else
      return( NULL );
   }




/* parse (argc, argv) argument list for morph */

static void  GetCLArgs( argc,argv,bname,fname,bands,factor )
   unsigned int argc;  /* count of arguments parsed by operating system    */
   char   *argv[];     /* pointers to arguments parsed by operating system */
   char   *bname;      /* input image files base name */
   char   *fname;      /* output file name */
   int    *bands;      /* number of bands in the spectrum */
   double *factor;     /* threshold multiplication factor */
   {
   int n = argc;
   int i,j;

   /* set args to default values */
   *bands  = 6;
   *factor = DFLTTMF;

   /* get arguments */
   strcpy( bname, argv[1] );
   strcpy( fname, argv[2] );
   *bands = atoi( argv[3] );

   /* get switch arguments */
   while ( --n )
      {
      if ( argv[n][0] == '-' )          /* then this is a switch */
         {
         if ( argv[n][1] == 'f' ) /* get threshold multiplication factor */
            {
            *factor = atof( argv[n+1] );
            }
         }    /* end of if ( argv[n][0] == '-' ) */
      }   /* end of  while ( --n ) "get arguments" loop */


   return;
   }



