/* mthresh.c -- program to threshold a grayscale image */
/* version 1.0     7 June 1994 */

/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */ 
/* Nashville, TN 37235                             */ 
/* rap2@vuse.vanderbilt.edu                        */ 
/*                                                 */ 
/* This software is freely redistributable if      */ 
/* the author's name and affiliation are included. */


#include <math.h>		
#include <stdio.h>
#include <string.h>
#include <values.h>
#include "rasterio.h"
#include "mthresh.h"


main( argc, argv ) 
   unsigned int argc;    /* count of arguments parsed by operating system    */
   char *argv[];         /* pointers to arguments parsed by operating system */
   {
   FILE *fp;                /* file pointer */
   struct rasterfile H;     /* rasterfile header */
   byte *cmap;              /* colormap */
   byte *In,*Out;           /* image pointers */
   byte *u,*v;              /* image pointers */
   int X,Y,N;               /* image dimensions */
   int i,j,k,n;             /* indices */
   int lower, upper;        /* threshold values */
   int moment;              /* T ==> use second moment */
   float factor;            /* threshold scale factor */


   if (argc < 3)
      {
      fprintf(stderr,
         "usage: mthresh image_in image_out [-l nnn] [-u nnn] [-m [nnn.nnn]]\n");
      exit(0);
      }

   lower = 128;
   upper = 255;
   moment = FALSE;
   factor = 0.0;

   /* get arguments */
   n = argc;
   while ( --n )
      {
      if ( argv[n][0] == '-' )          /* then this is a switch */
         {
         if ( argv[n][1] == 'l' )       /* lower threshold follows */
            {
            lower = atoi( argv[n+1] );
            }
         else if ( argv[n][1] == 'u' )  /* upper threshold follows */
            {
            upper = atoi( argv[n+1] );
            }
         else if ( argv[n][1] == 'm' )  /* use second moment of histo */
            {
            moment = TRUE;
            if ( n+2<argc && isdigit( argv[n+2][0] ) ) 
               {   /* then there is a scale factor */
               factor = atof( argv[n+2] );
               }
            }
         }    /* end of if ( argv[n][0] == '-' ) */
      }   /* end of  while ( --n ) "get arguments" loop */

   /* open input file */
   if ( !(fp = OpenFile( argv[1], "IMGPATHI", "r" )) ) exit( 0 );
   if (  ReadRasterFile( fp, &H, &cmap, &In, 0, 0, 0, 0, ALLOC )  ) exit( 0 );
   fclose( fp );


   /* remap the image */
   ExtractLuminance( &H, cmap, In, 0, 0, 0, 0 );


   /* image dimensions */
   X = H.ras_width;
   Y = H.ras_height;
   N = X * Y;

   /* make output image. */
   if ( !(Out = (byte *)calloc( N, sizeof(byte) )) )
         {
         fprintf(stderr, "Unable to allocate output image buffer.");
         exit( 0 );
         }

   /* compute the second moment */
   if ( moment )
      {
      lower = moment2( In, X, Y );
      if ( factor )
          lower = (int)( factor*(float)lower + 0.5 );
      }

   /* do the threshold */
   u = In;
   v = Out;
   for ( i=N; i; --i )
      {
      if ( (*u < lower) || (*u > upper) )
         *(v++) = BLACK;
      else
         *(v++) = WHITE;
      ++u;
      }
        
   /* create output file */
   if ( !(fp = OpenFile( argv[2], "IMGPATHO", "w" )) ) exit( 0 );

   WriteRasterFile( fp, &H, cmap, Out, 0, 0, 0, 0 ); 
               
   fclose( fp );
   free( In );
   free( Out );
   if ( cmap ) free( cmap );

   }
