/* gmf version 1.0   27 Apr 1992                   */
/* generalized morphology program                  */
/*                                                 */
/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */
/* Nashville, TN 37235                             */
/* rap2@vuse.vanderbilt.edu                        */
/*                                                 */
/* This software is freely redistributable if      */
/* the author's name and affiliation are included. */


#include "../2d/morph_sub.h"
#include "rasterio.h"

#define MIN(a,b) ((a<b)?(a):(b))
#define MAX(a,b) ((a>b)?(a):(b))


static   struct rasterfile RasHd;    /* raster file header */
static   unsigned char *cmap;        /* colormap */
 
main( argc, argv ) 
   unsigned int argc;    /* count of arguments parsed by operating system    */
   char *argv[];         /* pointers to arguments parsed by operating system */
   {

   byte *In,*I1,*I2,*I3,*I4,*Out;
   int i,p;
   int X,Y,M;
   FILE *fp;


   if (argc < 3 ) /* display usage */
      {
      fprintf(stderr, "usage: gmf InFile OutFile \n");
      exit(0);
      }


   /* open input file 1 */
   if ( !(fp = OpenFile( argv[1], "IMGPATHI", "r" )) ) exit( 0 );
   if (  ReadRasterFile( fp, &RasHd, &cmap, &In, 0, 0, 0, 0, ALLOC )  ) exit( 0 );
   fclose( fp );
   X = RasHd.ras_width;
   Y = RasHd.ras_height;
   M = X*Y;


   /* get the luminance */
   ExtractLuminance( &RasHd, cmap, In, 0, 0, 0, 0 );


   /* allocate image space */
   if ( !( I1 = (byte *)calloc( M, sizeof(byte) )) )
      {
      fprintf(stderr,"main: Unable to allocate image 1.\n");
      exit(0);
      }
   if ( !( I2 = (byte *)calloc( M, sizeof(byte) )) )
      {
      fprintf(stderr,"main: Unable to allocate image 2.\n");
      exit(0);
      }
   if ( !( I3 = (byte *)calloc( M, sizeof(byte) )) )
      {
      fprintf(stderr,"main: Unable to allocate image 3.\n");
      exit(0);
      }
   if ( !( I4 = (byte *)calloc( M, sizeof(byte) )) )
      {
      fprintf(stderr,"main: Unable to allocate image 4.\n");
      exit(0);
      }
   if ( !( Out = (byte *)calloc( M, sizeof(byte) )) )
      {
      fprintf(stderr,"main: Unable to allocate output image.\n");
      exit(0);
      }


   /* Do openings */
   MorphSub( OPEN, In, I1, X, Y, GRAIMG, FALSE, 0, 0, "k1.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( OPEN, In, I2, X, Y, GRAIMG, FALSE, 0, 0, "k2.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( OPEN, In, I3, X, Y, GRAIMG, FALSE, 0, 0, "k3.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( OPEN, In, I4, X, Y, GRAIMG, FALSE, 0, 0, "k4.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);

   /* Find Max */
   for ( i=M; i; --i )
       {
       p = MAX(*(I1+i),*(I2+i));
       p = MAX(p,*(I3+i));
       p = MAX(p,*(I4+i));
       *(Out+i) = (byte)p;
       }

   /* Do closings */
   MorphSub( CLOSE, Out, I1, X, Y, GRAIMG, FALSE, 0, 0, "k1.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( CLOSE, Out, I2, X, Y, GRAIMG, FALSE, 0, 0, "k2.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( CLOSE, Out, I3, X, Y, GRAIMG, FALSE, 0, 0, "k3.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( CLOSE, Out, I4, X, Y, GRAIMG, FALSE, 0, 0, "k4.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);

   /* Find Min */
   for ( i=M; i; --i )
       {
       p = MIN(*(I1+i),*(I2+i));
       p = MIN(p,*(I3+i));
       p = MIN(p,*(I4+i));
       *(Out+i) = (byte)p;
       }

   /* Do closings */
   MorphSub( CLOSE, In, I1, X, Y, GRAIMG, FALSE, 0, 0, "k1.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( CLOSE, In, I2, X, Y, GRAIMG, FALSE, 0, 0, "k2.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( CLOSE, In, I3, X, Y, GRAIMG, FALSE, 0, 0, "k3.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( CLOSE, In, I4, X, Y, GRAIMG, FALSE, 0, 0, "k4.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);

   /* Find Min */
   for ( i=M; i; --i )
       {
       p = MIN(*(I1+i),*(I2+i));
       p = MIN(p,*(I3+i));
       p = MIN(p,*(I4+i));
       *(In+i) = (byte)p;
       }

   /* Do openings */
   MorphSub( OPEN, In, I1, X, Y, GRAIMG, FALSE, 0, 0, "k1.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( OPEN, In, I2, X, Y, GRAIMG, FALSE, 0, 0, "k2.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( OPEN, In, I3, X, Y, GRAIMG, FALSE, 0, 0, "k3.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);
   MorphSub( OPEN, In, I4, X, Y, GRAIMG, FALSE, 0, 0, "k4.tfm",
                NULL, ZERO, 0, 0, 0, GRASE, SET, 0, FALSE, FALSE);

   /* Find Max */
   for ( i=M; i; --i )
       {
       p = MAX(*(I1+i),*(I2+i));
       p = MAX(p,*(I3+i));
       p = MAX(p,*(I4+i));
       *(In+i) = (byte)p;
       }

   /* average OLCL and CLOL */
   for ( i=M; i; --i )
       {
       p = *(In+i) + *(Out+i);
       *(Out+i) = (byte)(p >> 1);
       }

   /* write output file */
   if ( !(fp = fopen( argv[2], "w" )) )
      {
      fprintf( stderr, "unable to open %s\n",argv[2]);
      exit(0);
      }
   WriteRasterFile( fp, RasHd, cmap, Out, 0, 0, 0, 0 );
   fflush( fp );
   fclose( fp );

   free(In);
   free(I1);
   free(I2);
   free(I3);
   free(I4);
   free(Out);
   if ( cmap ) free(cmap);
   }






