/* constras.c  --  program to generate a constant rasterfile. */

/* version 1.0     7 June 1994 */

/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */ 
/* Nashville, TN 37235                             */ 
/* rap2@vuse.vanderbilt.edu                        */ 
/*                                                 */ 
/* This software is freely redistributable if      */ 
/* the author's name and affiliation are included. */

#include "stdio.h"
#include "malloc.h"
#include "rasterio.h"

#define BLACK   0
#define WHITE 255


main( argc, argv )
   unsigned int argc;
   char * argv[];
   {
   struct rasterfile *RasHd;
   byte *cmap;
   byte *u,*Img;
   int val = BLACK;
   int n = argc;
   int X = 512;
   int Y = 512;
   int N;


   /* check args */

   if ( argc > 1  && argv[1][0] == '?' )
      {
      fprintf(stderr, "usage:  constras > rasterfile [-C nnn] [-X nnn] [-Y nnn]\n");
      exit(0);
      }


   /* get args */

   while ( --n )
      {
      if ( argv[n][0] == '-' )          /* then this is a switch */
         {
         if ( argv[n][1] == 'C' )  
            {
            val = atoi( argv[n+1] );
            if ( (val < BLACK) || (val > WHITE) )
               {
               fprintf(stderr, "Constant must be in [%d,%d].\n",BLACK,WHITE);
               exit(0);
               }
            }
         else if ( argv[n][1] == 'X' )  
            X = atoi( argv[n+1] );
         else if ( argv[n][1] == 'Y' )  
            Y = atoi( argv[n+1] );
         else
            {
            fprintf(stderr, "Invalid argument\n");
            exit(0);
            }
         }
      }


   /* allocate image */
   N = X * Y;
   if ( !(Img = (byte *)calloc( N, sizeof(byte) )) )
      {
      fprintf(stderr,"Unable to allocate space for image\n");
      exit(0);
      }

   /* if the constant is not zero */
   if ( val )
      memset( Img, val, N );

   /* make and write rasterfile */

   RasHd = MakeHeader( X, Y );
   cmap  = MakeColorMap();

   WriteRasterFile( stdout, RasHd, cmap, Img, 0, 0, 0, 0 ); 

   free( RasHd );
   free( cmap  );
   }




 
