/*======================================================================
 * pmtbs - object to store and operate on multi-channel, passive 
 *         microwave TB arrays
 *
 * 09/09/1997 brodzik brodzik@zamboni.colorado.edu 303-492-8263
 * National Snow & Ice Data Center, University of Colorado, Boulder
 *
 *$Revision: 1.8 $
 *
 *======================================================================*/
#ifndef pmtbs_h_
#define pmtbs_h_

#include "define.h"
#include "grids.h"
#include "pmalgorithm.h"

static const char pmtbs_h_rcsid[] = "$Header: /tmp_mnt/FILES/pmalgos/pmtbs.h,v 1.8 1999-08-17 16:38:41 brodzik Exp $";

/*
 * Multi-channal TB arrays structure
 *
 * TB data array is defined as a fixed array of pointers, one for each
 * defined channel, even though only the channels needed for the specified
 * algorithm will actually have matrices allocated at any one time.
 *
 */
typedef struct {
  grid_class *grid;              /* grid information */
  char *tbs_basename;            /* basename (no extension) of files to read */
  pmalgo_class *algorithm;       /* algorithm information */
  byte2 **data[PMALGO_NUM_CHANNELS];
                                 /* array of pointers to TB matrices for 
				  * each channel, only pointers to channels 
				  * needed for the current algorithm are used
				  */
} pmtbs_class;

/* Function prototypes */
void turn_on_new_extensions();
void turn_off_new_extensions();
bool new_channels();
char* pmalgo_extension_format(int channel);
char* pmalgo_channel_extension(int channel);



/* Use pmtbs->algorithm->run to run the current algorithm on a pixel. */
pmtbs_class *init_pmtbs(grid_class *grid,
			char *algorithm_name,
                        char *platform_name,
			char *tbs_basename,
			bool verbose);
bool write_pmtbs(pmtbs_class *this, char *tbs_basename, bool verbose);
void close_pmtbs(pmtbs_class *this);

#endif
