/*======================================================================
 * snow_gb - calculate passive microwave-derived snow/surface 
 *           classification of Norm Grody and Alan Basist.
 *
 * 06/12/2000 brodzik brodzik@zamboni.colorado.edu 303-492-8263
 * National Snow & Ice Data Center, University of Colorado, Boulder
 *
 *======================================================================*/
#ifndef snow_gb_h_
#define snow_gb_h_

#include "define.h"

static const char snow_gb_h_rcsid[] = "$Header: /tmp_mnt/FILES/pmalgos/snow_gb.h,v 1.7 2004-06-28 19:00:38 brodzik Exp $";

/* general algorithm return codes */
typedef enum {
  GB_NO_SNOW, GB_GLACIER_ICE, GB_SNOW
} gb_return_code;

/* specific algorithm return codes */
typedef enum {
  /* No snow categories: */
  GB_PC1, GB_PC2, GB_CDS, GB_FZG, GB_UNK,
  /* Glacier ice category: */
  GB_GIC,
  /* Snow categories: */
  GB_MX1, GB_SNW, 
  GB_NUM_SURFACE_CONDITIONS
} gb_surface_condition;

/* 
 * Most of these strings match original implementation's 
 * "setcond" condition strings 
 */
static const char *gb_surface_condition_code[] = {
  "PC1", "PC2", "CDS", "FZG", "UNK", "GIC",
  "MX1", "SNW"
};
static const char *gb_surface_condition_name[] = {
  "Precipitation1", "Precipitation2", "Cold Deserts",
  "Frozen Ground", "Unknown", 
  "Subsurface Ice",
  "Mixed Pixels", "Snow"
};

gb_return_code snow_gb(gb_surface_condition *surface_condition, 
		       byte2 tb19v, byte2 tb19h, 
		       byte2 tb22v, 
		       byte2 tb37v,
		       byte2 tb85v, byte2 tb85h,
		       int verbose);


#endif
