#!/usr/local/bin/perl -w
#
# This perl script will test the C program pmalgos for the Chang
# surface algorithm
#
#$Log: not supported by cvs2svn $
#
#-----------------------------------------------------------------------
#Usage: test_pmalgos_csurf.pl (no parameters)
#

# force autoflush after every write or print
$| = 1;
if (-1 >= $#ARGV) {
    print "Usage: test_pmalgos_csurf.pl case#'s\n";
    print "Example: test_pmalgos_csurf.pl 1\n";
    exit;
}

$topdir = "/usr/people/brodzik/";
$ancillary_dir = "${topdir}ancillary";
$testdir = "${topdir}src/pmalgos/test";
$comparedir = "${topdir}src/pmalgos/test/compare";

$pmalgos = "${topdir}src/pmalgos/pmalgos";
$fdistill = "${topdir}bin/fdistill";
$forest = "${ancillary_dir}/NL.albedo_90N-90S_0-360E";
$glcc_forest = "${ancillary_dir}/Nligbp01.dat.max50.float";
$landmask = "${ancillary_dir}/NL.FlagsMask_90N-90S_0-360E";
$mm_landmask = "${ancillary_dir}/NL.FlagsMask_90N-90S_0-360E.12";
$landmask11 = "${ancillary_dir}/NL.FlagsMask_90N-90S_0-360E.11";
$tb_basename = '/EASE/pmalgos_test/EASE-F13-NL1999354D';
$alt_tb_basename = '/EASE/pmalgos_test/EASE-F11-NL1993312D';
$month = 12;
$alt_month = 11;
$gpd = 'Nl.gpd';

while (defined($testcase = shift)) {

    if (1 == $testcase) {
	print "Case 1: missing landmask\n";
	print "Expected result: error message, and EXIT_FAILURE\n";
	`$pmalgos -t $forest $gpd CHANG_SURFACE $tb_basename`;
	if (0 == ($? >> 8)) {
	    die "ERROR: return was not EXIT_FAILURE\n";
	} else {
	    print "SUCCESS: returned EXIT_FAILURE\n";
	    print "------------------------------------------------\n\n";
	}
    } elsif (2 == $testcase) {
	print "Case 2: invalid option -s\n";
	print "Expected result: error message, usage message about invalid option, and EXIT_FAILURE\n";
	`$pmalgos -v -m $landmask -s bogus -t $forest $gpd CHANG_SURFACE $tb_basename`;
	if (0 == ($? >> 8)) {
	    die "ERROR: return was not EXIT_FAILURE\n";
	} else {
	    print "SUCCESS: returned EXIT_FAILURE\n";
	    print "------------------------------------------------\n\n";
	}
    } elsif (3 == $testcase) {
	print "Case 3: wrong-sized forest cover file\n";
	print "Expected result: error message, usage message, and EXIT_FAILURE\n";
	`$pmalgos -v -m $landmask -t $landmask $gpd CHANG_SURFACE $tb_basename`;
	if (0 == ($? >> 8)) {
	    die "ERROR: return was not EXIT_FAILURE\n";
	} else {
	    print "SUCCESS: returned EXIT_FAILURE\n";
	    print "------------------------------------------------\n\n";
	}
    } elsif (4 == $testcase) {
	print "Case 4: legitimate call, for CHANG_SURFACE\n";
	print "Expected result: output file created, EXIT_SUCCESS\n";
	$outfile = $testdir . "/Nl.test_pmalgos_csurf.case" . $testcase;
	$outfile_byte1 = $outfile . ".byte1";
	`$pmalgos -v -f 255 -o $outfile -m $landmask -t $forest $gpd CHANG_SURFACE $tb_basename`;
	if (0 != $?) {
	    die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
	} else {
	    print "SUCCESS: returned EXIT_SUCCESS\n";
	    $compare_file = sprintf("%s/Nl.test_pmalgos_csurf.case%d.orig",
				    $comparedir,$testcase);
	    print "test    file is $outfile\n";
	    print "archive file is $compare_file\n";
	    `diff -s $outfile $compare_file`;
	    if (0 != $?) {
		die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		    ($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
	    } else {
		print "compare successful, fdistilling test file $outfile for next case\n\n";
		`$fdistill -vub $outfile > $outfile_byte1`;
		unlink $outfile;
	    }
	}
    } elsif (5 == $testcase) {
	print "Case 5: legitimate call, for CHANG_DEPTH\n";
	print "Expected result: output file created, EXIT_SUCCESS\n";
	$outfile = $testdir . "/Nl.test_pmalgos_csurf.case" . $testcase;
	$surface_byte1 = $testdir . "/Nl.test_pmalgos_csurf.case4.byte1";
	`$pmalgos -v -f -3 -o $outfile -h $month -m $landmask -g $surface_byte1 -t $forest $gpd CHANG_DEPTH $tb_basename`;
	if (0 != $?) {
	    die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
	} else {
	    print "SUCCESS: returned EXIT_SUCCESS\n";
	    $compare_file = sprintf("%s/Nl.test_pmalgos_csurf.case%d.orig",
				    $comparedir,$testcase);
	    print "test    file is $outfile\n";
	    print "archive file is $compare_file\n";
	    `diff -s $outfile $compare_file`;
	    if (0 != $?) {
		die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		    ($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
	    } else {
		print "compare successful, deleting test file $outfile\n\n";
		unlink $outfile;
		print "...and deleting case 4 testfile $surface_byte1\n";
		unlink $surface_byte1;
	    }
	}
    } elsif (6 == $testcase) {
	print "Case 6: legitimate call, for NSIDC_SURFACE\n";
	print "Expected result: output file created, EXIT_SUCCESS\n";
	$outfile = $testdir . "/Nl.test_pmalgos_csurf.case" . $testcase;
	$outfile_byte1 = $outfile . ".byte1";
	`$pmalgos -v -f 255 -o $outfile -m $landmask11 -t $glcc_forest $gpd NSIDC_SURFACE $alt_tb_basename`;
	if (0 != $?) {
	    die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
	} else {
	    print "SUCCESS: returned EXIT_SUCCESS\n";
	    $compare_file = sprintf("%s/Nl.test_pmalgos_csurf.case%d.orig",
				    $comparedir,$testcase);
	    print "test    file is $outfile\n";
 	    print "archive file is $compare_file\n";
 	    `diff -s $outfile $compare_file`;
 	    if (0 != $?) {
 		die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		    ($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
 	    } else {
		print "compare successful, fdistilling test file $outfile for next case\n\n";
		`$fdistill -vub $outfile > $outfile_byte1`;
		unlink $outfile;
 	    }
	}
    } elsif (7 == $testcase) {
	print "Case 7: legitimate call, for NSIDC_S3_SWE\n";
	print "Expected result: output file created, EXIT_SUCCESS\n";
	$outfile = $testdir . "/Nl.test_pmalgos_csurf.case" . $testcase;
	$surface_byte1 = $testdir . "/Nl.test_pmalgos_csurf.case6.byte1";
	`$pmalgos -v -f -3 -o $outfile -h $alt_month -m $landmask11 -g $surface_byte1 -t $glcc_forest $gpd NSIDC_S3_SWE $alt_tb_basename`;
	if (0 != $?) {
	    die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
	} else {
	    print "SUCCESS: returned EXIT_SUCCESS\n";
	    $compare_file = sprintf("%s/Nl.test_pmalgos_csurf.case%d.orig",
				    $comparedir,$testcase);
	    print "test    file is $outfile\n";
	    print "archive file is $compare_file\n";
	    `diff -s $outfile $compare_file`;
	    if (0 != $?) {
		die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
		    ($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
	    } else {
		print "compare successful, deleting test file $outfile\n\n";
		unlink $outfile;
		print "...and deleting case 6 testfile $surface_byte1\n";
		unlink $surface_byte1;
	    }
	}
    }
}

1;




