#!/usr/local/bin/perl -w
#
# This perl script will test the C program pmalgos for the
# Nasateam algorithms
#
# 2000-02-17 M. J. Brodzik 303-492-8263 brodzik@nsidc.org
# National Snow & Ice Data Center, Boulder, CO
# Copyright (C) 2000-2006 University of Colorado
#
#$Id: test_pmalgos_nasateam.pl,v 1.7.2.1 2008-06-10 20:36:35 brodzik Exp $
#
#-----------------------------------------------------------------------
#Usage: test_pmalgos_nasateam.pl (no parameters)
#
use Env;
use lib ("${PERL_UTIL_PATH}");
use NSIDC_Unit_Test_Util;
use File::Temp qw/ tempdir /;
use File::Basename qw/ basename /;
use File::Spec::Functions;
use Getopt::Std;
sub tmp_test_file_name($$$);

# force autoflush after every write or print
$| = 1;
my $usage = '

test_pmalgos_nasateam.pl [-t test_binary]

Input: n/a

Options:
    t test_binary - pmalgos binary to test, default is
      ./pmalgos

';

undef %args;
if ( !getopts( 't:', \%args ) ) {
	printf STDERR $usage;
	exit;
}

if ( !$args{t} ) {
	$args{t} = "./pmalgos";
}

$tmp_test_dir          = tempdir();
$pmalgos_test_data_dir = "/share/data/pmalgos_test_data";
$map_dir               = ${pmalgos_test_data_dir} . "/maps";
$ancillary_dir         = ${pmalgos_test_data_dir} . "/ancillary";
$pmalgos               = "$args{t} -r -u 0 ";
$pmalgos_ns            = $args{t};                                  # _ns = "no switches"

# Use a call to the unix function hostname to determine where we are running.
# Do not use $ENV{HOSTNAME}, it returns unreliable results, depending on the
# user's shell.
my $host = `hostname -s`;
chomp($host);
$fdistill = "$ENV{FDISTILL_DIR}/fdistill -u ";

$tbsdir         = "${pmalgos_test_data_dir}/bootstrap";
$testdir        = "${pmalgos_test_data_dir}/";
@basename_list  = ( $tbsdir . "/tbs/N/N3B.990531", $tbsdir . "/tbs/S/S3B.981101" );
$goddard_tbsdir = "${pmalgos_test_data_dir}/tbs/goddard/f17";

@f17_basename_list = (
					   $goddard_tbsdir . "/tb_f17_20080915_nrt.N",
					   $goddard_tbsdir . "/tb_f17_20080915_nrt.S",
					   $goddard_tbsdir . "/tb_f17_20090315_nrt.N",
					   $goddard_tbsdir . "/tb_f17_20090315_nrt.S"
);

$goddard_climdir = "/share/data/nise/ice_clims";
@f17_clim_list = (
				   $goddard_climdir . "/n/050720/oceanmask.sep.1979-2003.n",
				   $goddard_climdir . "/s/050720/oceanmask.sep.1979-2003.s",
				   $goddard_climdir . "/n/050720/oceanmask.mar.1979-2003.n",
				   $goddard_climdir . "/s/050720/oceanmask.mar.1979-2003.s"
);
$goddard_comparedir = "${pmalgos_test_data_dir}/goddard/compare_ice";

# These were the files from Walt that I compared to.  Differences are summarized at
# http://cires.colorado.edu/~brodzik/pmalgos-F17/
@f17_compare_file = (
					  $goddard_comparedir . "/N/tb_f17_20080915_nrt.N.tot",
					  $goddard_comparedir . "/S/tb_f17_20080915_nrt.S.tot",
					  $goddard_comparedir . "/N/tb_f17_20090315_nrt.N.tot",
					  $goddard_comparedir . "/S/tb_f17_20090315_nrt.S.tot"
);

@platforms = ( F08, F11, F13, F15_BRIDGE );    # F17 will be tested on its own
%fill_value = (
				NASATEAM => 157,
				GODDARD  => 251,
);
%landmasks = (
			   N => $ancillary_dir . "/landmask-n3b.byte",
			   S => $ancillary_dir . "/landmask-s3b.byte",
);
%weightmasks = (
				 N => $ancillary_dir . "/cmin_goddard_1992.N11.byte",
				 S => $ancillary_dir . "/cmin_goddard_1992.S11.byte",
);

# Note:  Add map_dir to environment setting for PATHMPP
# for {loci_no_path} values to work
$ENV{"PATHMPP"} = $ENV{"PATHMPP"} . ":${map_dir}";

my $description;

# Test NASATEAM for each basename...
foreach $base (@basename_list) {

	$algo  = "NASATEAM";
	@parts = split( '3B\.', $base );
	$hem   = chop( $parts[0] );
	$lchem = lc($hem);

	$test_file    = tmp_test_file_name( $tmp_test_file_name, $base, ".${algo}.tot" );
	$compare_file = $testdir . lc( ${algo} ) . '/compare_ice/' . $hem . '/' . $parts[1] . '.tot';
	$description  = "\ntest_pmalgos_nasateam ${algo}_${hem}_SEAICE for basename $base\n" . "using pmalgos at $pmalgos\n";

	$cmd = "$pmalgos -f $fill_value{$algo} -l ${ancillary_dir}/t_mask.${lchem}3b";
	$cmd = "$cmd ${hem}3B.gpd ${algo}_${hem}_SEAICE $base";
	print "Calling: $cmd | $fdistill > $test_file\n";
	`$cmd | $fdistill > $test_file`;
	check_for_success( $description, $?, $test_file, $compare_file );

	if ( "N" eq $hem ) {

		# Check multi-year ice also, for Northern hemisphere
		$test_file    = tmp_test_file_name( $tmp_test_dir, $base, ".${algo}.myr" );
		$compare_file = $testdir . lc( ${algo} ) . '/compare_ice/' . $hem . '/' . $parts[1] . '.myr';
		$description  = "\ntest_pmalgos_nasateam ${algo}_${hem}_MYICE for basename $base\n" . "using pmalgos at $pmalgos\n";

		$cmd = "$pmalgos -f $fill_value{$algo} -l ${ancillary_dir}/t_mask.${lchem}3b";
		$cmd = "$cmd ${hem}3B.gpd ${algo}_${hem}_MYICE $base";
		print "Calling: $cmd | $fdistill > $test_file\n";
		`$cmd | $fdistill > $test_file`;
		check_for_success( $description, $?, $test_file, $compare_file );
	}
}

# Test Goddard Time Series for each basename...
foreach $base (@basename_list) {


	$algo  = "GODDARD";
	@parts = split( '3B\.', $base );
	$hem   = chop( $parts[0] );
	$lchem = lc($hem);

	printf STDERR "Next basename  = $base\n";

	foreach $platform (@platforms) {

		$test_file    = tmp_test_file_name( $tmp_test_dir, $base, ".${algo}.tot" );
		$compare_file = $testdir . lc( ${algo} ) . '/compare_ice/' . $hem . '/' . $hem . "3B." . $platform . "." . $parts[1] . '.GODDARD.tot';
		$description  = "\ntest_pmalgos_nasateam $platform ${algo}_SEAICE for basename $base\n" . "using pmalgos at $pmalgos\n";

		$cmd = "$pmalgos  -f $fill_value{$algo} -l ${ancillary_dir}/t_mask.${lchem}3b -q ${hem} -p $platform ";
		$cmd = "$cmd ${hem}3B.gpd ${algo}_SEAICE $base";
		print "Calling: $cmd | $fdistill > $test_file \n";
		`$cmd | $fdistill > $test_file`;
		check_for_success( $description, $?, $test_file, $compare_file );

		if ( "N" eq $hem ) {

			# Check multi-year ice also, for Northern hemisphere only
			$test_file    = tmp_test_file_name( $tmp_test_dir, $base, ".${algo}.myr" );
			$compare_file = $testdir . lc( ${algo} ) . '/compare_ice/' . $hem . '/' . $hem . "3B." . $platform . "." . $parts[1] . '.GODDARD.myr';
			$description  = "\ntest_pmalgos_nasateam $platform ${algo}_MYICE for basename $base\n" . "using pmalgos at $pmalgos\n";

			$cmd = "$pmalgos  -f $fill_value{$algo} -l ${ancillary_dir}/t_mask.${lchem}3b -q ${hem} -p $platform ";
			$cmd = "$cmd ${hem}3B.gpd ${algo}_MYICE $base";
			print "Calling: $cmd | $fdistill > $test_file\n";
			`$cmd | $fdistill > $test_file`;
			check_for_success( $description, $?, $test_file, $compare_file );

		}
	}
}

# Test Goddard Time Series with output scaling in percent for each basename...
foreach $base (@basename_list) {

	$algo  = "GODDARD";
	@parts = split( '3B\.', $base );
	$hem   = chop( $parts[0] );
	$lchem = lc($hem);

	$platform = F15_BRIDGE;

	$test_file    = tmp_test_file_name( $tmp_test_dir, $base, ".${algo}.pcent.tot" );
	$compare_file = $testdir . lc( ${algo} ) . '/compare_ice/' . $hem . '/' . $hem . "3B." . $platform . "." . $parts[1] . '.GODDARD.pcent.tot';
	$description  = "\ntest_pmalgos_nasateam $platform ${algo}_SEAICE output in percent for basename $base\n" . "using pmalgos at $pmalgos\n";

	$cmd = "$pmalgos  -z -f $fill_value{$algo} -l ${ancillary_dir}/t_mask.${lchem}3b -q ${hem} -p $platform ";
	$cmd = "$cmd ${hem}3B.gpd ${algo}_SEAICE $base";
	print "Calling: $cmd  | $fdistill > $test_file\n";
	`$cmd | $fdistill > $test_file`;
	check_for_success( $description, $?, $test_file, $compare_file );

	if ( "N" eq $hem ) {

		# Check multi-year ice also, for Northern hemisphere only
		$test_file    = tmp_test_file_name( $tmp_test_dir, $base, ".${algo}.pcent.myr" );
		$compare_file = $testdir . lc( ${algo} ) . '/compare_ice/' . $hem . '/' . $hem . "3B." . $platform . "." . $parts[1] . '.GODDARD.pcent.myr';
		$description  = "\ntest_pmalgos_nasateam $platform ${algo}_MYICE output in percent for basename $base\n" . "using pmalgos at $pmalgos\n";

		$cmd = "$pmalgos  -z -f $fill_value{$algo} -l ${ancillary_dir}/t_mask.${lchem}3b -q ${hem} -p $platform ";
		$cmd = "$cmd ${hem}3B.gpd ${algo}_MYICE $base";
		print "Calling: $cmd  | $fdistill > $test_file\n";
		`$cmd | $fdistill > $test_file`;
		check_for_success( $description, $?, $test_file, $compare_file );

	}
}

# Test Goddard Time Series error conditions
$algo        = GODDARD;
$platform    = F15_BRIDGE;
$landmask    = 'boguslandmask';
$weightmask  = 'bogusweightmask';
$hem         = 'N';
$lchem       = lc($hem);
$base        = $basename_list[0];
$description = "\nGODDARD_SEAICE error condition: -j options but missing landmask...\n";
$cmd         = "$pmalgos -j -q ${hem} -p $platform -w $weightmask";
$cmd         = "$cmd ${hem}3B.gpd ${algo}_SEAICE $base";
print "Calling: $cmd\n";
`$cmd`;
check_for_failure( $description, $? );

$description = "\nGODDARD_SEAICE error condition: -j options but missing weightsmask...\n";
$cmd         = "$pmalgos -j -q ${hem} -p $platform -m $landmask";
$cmd         = "$cmd ${hem}3B.gpd ${algo}_SEAICE $base";
print "Calling: $cmd\n";
`$cmd`;
check_for_failure( $description, $? );

printf STDERR "Doing F17 tests...\n";

# Original test Goddard Time Series for F17 data was originally to files from Walt (no scaling)
# Once I understood those differences, I saved the output here for exact difference checks in
# future runs.
for ( $i = 0; $i < @f17_basename_list; $i++ ) {

	$base         = $f17_basename_list[$i];
	$clim_file    = $f17_clim_list[$i];
	$test_file    = tmp_test_file_name( $tmp_test_dir, $base, '.tot' );
	$compare_file = $f17_compare_file[$i];


	$platform = F17;
	$algo     = "GODDARD";
	@parts    = split( '\.', $base );
	$hem      = chop( $parts[1] );
	$lchem    = lc($hem);

	$description = "\ntest_pmalgos_nasateam $platform ${algo}_SEAICE output no scaling for basename $base\n" . "using pmalgos at $pmalgos\n";

	#    $cmd = "$pmalgos -f $fill_value{$algo} -c $clim_file -l ${ancillary_dir}/lmask_expanded.${lchem} -q ${hem} -p $platform ";
	$cmd = "$pmalgos  -c $clim_file -l ${ancillary_dir}/lmask_expanded.${lchem} -q ${hem} -p $platform ";
	$cmd = "$cmd ${hem}3B.gpd ${algo}_SEAICE $base";
	print "Calling: $cmd | $fdistill > $test_file\n";
	`$cmd | $fdistill > $test_file`;
	check_for_success( $description, $?, $test_file, $compare_file );

}

printf STDERR "Doing spillover correction tests...\n";

# Comparing output before and after spillover correction with output from
# NRTSI processing.  NRTSI processing currently implements the spillover correction
# as an IDL routine.  This set of tests compares the IDL spillover correction
# implementation with the new pmalgos C implementation.
# The NRTSI files for compareison are these:
# @spillover_compare_list = ($nrtsi_tbsdir . "/nt_20090417_f13_nrt_n.bin.noheader",
#                           $nrtsi_tbsdir . "/nt_20090417_f13_nrt_s.bin.noheader",);
# Once I could explain the last remaining differences on these, I saved
# pmalgos output for regression testing here.

# TODO [MHS, 2013-01-18] - /projects/EASE is corrupted and we need to regenerate data for spillover tests.


# $nrtsi_tbsdir = "/projects/EASE/pmalgos_test/spillover";
# @basename_list = ($nrtsi_tbsdir . "/NRTSI_SSMIF13_20090417_N",
#                   $nrtsi_tbsdir . "/NRTSI_SSMIF13_20090417_S",);
# @no_spillover_compare_list = ($nrtsi_tbsdir . "/NRTSI_SSMIF13_20090417_N.BIN",
#                               $nrtsi_tbsdir . "/NRTSI_SSMIF13_20090417_S.BIN",);
# @spillover_compare_list = ($goddard_comparedir . "/N/NRTSI_SSMIF13_20090417_N.spillover.tot",
#                            $goddard_comparedir . "/S/NRTSI_SSMIF13_20090417_S.spillover.tot", );
# for ( $i=0; $i < @basename_list; $i++ ) {

#     $base = $basename_list[$i];

#     $platform = F13;
#     $algo = "GODDARD_SEAICE";
#     @parts = split('\.',$base);
#     $hem = chop(substr($parts[0],length($parts[0])-1,1));
#     $lchem = lc($hem);
# #    $landmask = $landmasks{$hem};
# #    $weightmask = $weightmasks{$hem};

#     $test_file = tmp_test_file_name($tmp_test_dir, $base, '.nospillover.tot');
#     $compare_file = $no_spillover_compare_list[$i];

#     $description = "\ntest_pmalgos_nasateam $platform ${algo} no spillover for basename $base\n"
# 	. "using pmalgos at $pmalgos\n";

#     $cmd = "$pmalgos -r -f 255 -q ${hem} -p $platform ";
#     $cmd = "$cmd ${hem}3B.gpd ${algo} $base";
#     print "Calling: $cmd | $fdistill -u > $test_file\n";
#     `$cmd | $fdistill -u > $test_file`;
#     check_for_success( $description,
# 		       $?,
# 		       $test_file,
# 		       $compare_file );

#     # Now do the same processing, but turn on spillover and compare to NRTSI final
#     $test_file = tmp_test_file_name($tmp_test_dir, $base, '.spillover.tot');
#     $compare_file = $spillover_compare_list[$i];
#     $description = "\ntest_pmalgos_nasateam $platform ${algo} w/spillover for basename $base\n"
# 	. "using pmalgos at $pmalgos\n";

#     $cmd = "$pmalgos -j -x 15 -m $landmasks{$hem} -w $weightmasks{$hem} -r -f 255 -q ${hem} -p $platform ";
#     $cmd = "$cmd -l /share/data/seaice_index/ancillary/pmalgos/oceanmask.${hem}3B.locp -u 0";
#     $cmd = "$cmd -c /share/data/seaice_index/ancillary/pmalgos/oceanmask.apr.1979-2002.$lchem.ice_clim";
#     $cmd = "$cmd ${hem}3B.gpd ${algo} $base";
#     print "Calling: $cmd | $fdistill -u > $test_file\n";
#     `$cmd | $fdistill -u > $test_file`;
#     check_for_success( $description,
# 		       $?,
# 		       $test_file,
# 		       $compare_file );

# }



# Running GODDARD_SEAICE spillover (for NISE processing)
# this is the original set of tests, that does thresholding on seaice unconditionally
$nise_tbsdir    = "${pmalgos_test_data_dir}/tbs";
$nise_climdir   = "/share/data/nise/ice_clims/NISE-107";
$nise_ancillary = "/share/data/nise/ancillary";
$compare_dir    = "${pmalgos_test_data_dir}/goddard/compare_ice";


# [MHS, 2013-01-29] in order to test GODDARD_PUB_SEAICE (really just the F17_final tiepoints)
# we need to add a new set of comparison files and extensions to the test files.
# I've decided that adding super loops and repeating the initial tests would work fine (again ignoring the f13 TBS)

# NISE TB files from 2009-06-05
# Technically these are F13 TBs, but that's irrelevant to this test
@basename_list = ( 'Nl_ditb_4711', 'Sl_ditb_4699', 'Nl_ditb_4711', 'Sl_ditb_4699', 'Nl_ditb_4711', 'Sl_ditb_4699', );
@compare_list = (
				  $compare_dir . '/N/Nl_ditb_4711.no_spillover.tot',
				  $compare_dir . '/S/Sl_ditb_4699.no_spillover.tot',
				  $compare_dir . '/N/Nl_ditb_4711.spillover.tot',
				  $compare_dir . '/S/Sl_ditb_4699.spillover.tot',
				  $compare_dir . '/N/Nl_ditb_4711.spillover.no_threshold.tot',
				  $compare_dir . '/S/Sl_ditb_4699.spillover.no_threshold.tot',
);

@goddard_pub_compare_list = (
							  $compare_dir . '/N/Nl_ditb_4711.PUB.no_spillover.tot',
							  $compare_dir . '/S/Sl_ditb_4699.PUB.no_spillover.tot',
							  $compare_dir . '/N/Nl_ditb_4711.PUB.spillover.tot',
							  $compare_dir . '/S/Sl_ditb_4699.PUB.spillover.tot',
							  $compare_dir . '/N/Nl_ditb_4711.PUB.spillover.no_threshold.tot',
							  $compare_dir . '/S/Sl_ditb_4699.PUB.spillover.no_threshold.tot',
);

@ext = ( '.no_spillover.tot',
         '.no_spillover.tot',
		 '.spillover.threshold.tot',
         '.spillover.threshold.tot',
		 '.spillover.no_threshold.tot',
         '.spillover.no_threshold.tot', );

@goddard_pub_ext = ( '.PUB.no_spillover.tot',
                     '.PUB.no_spillover.tot',
                     '.PUB.spillover.threshold.tot',
                     '.PUB.spillover.threshold.tot',
                     '.PUB.spillover.no_threshold.tot',
                     '.PUB.spillover.no_threshold.tot', );

@cmds = (
		  "$pmalgos_ns -z -f 255 -q HEM -p PLATFORM -c CLIMFILE ",
		  "$pmalgos_ns -z -f 255 -q HEM -p PLATFORM -c CLIMFILE ",
		  "$pmalgos_ns -zj -x 15 -m LANDMASK -w WEIGHTMASK -f 255 -q HEM -p PLATFORM -c CLIMFILE ",
		  "$pmalgos_ns -zj -x 15 -m LANDMASK -w WEIGHTMASK -f 255 -q HEM -p PLATFORM -c CLIMFILE ",
		  "$pmalgos_ns -zj -m LANDMASK -w WEIGHTMASK -f 255 -q HEM -p PLATFORM -c CLIMFILE ",
		  "$pmalgos_ns -zj -m LANDMASK -w WEIGHTMASK -f 255 -q HEM -p PLATFORM -c CLIMFILE ",
);

@clim_month = ( 6, 6, 6, 6, 6, 6, );


# Want to run F17 tests over old and new regression values (remember that
# GODDARD_PUB_SEAICE => is F17_final in tiepoints language) So set up lists
# of compare and extentions here.


@test_algorithms = ( "GODDARD_SEAICE", "GODDARD_PUB_SEAICE" );
@compare_lists = ( \@compare_list, \@goddard_pub_compare_list );
@ext_lists     = ( \@ext,          \@goddard_pub_ext );

# Loop over each algorithm substituting in the comparision and extension lists.
for (my $a_counter = 0; $a_counter <= $#test_algorithms; $a_counter++){

   $algorithm = $test_algorithms[$a_counter];
   printf STDERR "Doing F17 tests for $algorithm...\n";

   @current_compare_list = @{$compare_lists[$a_counter]};
   @current_ext = @{$ext_lists[$a_counter]};

   for ( $i = 0; $i < @basename_list; $i++ ) {

      $hem      = substr( $basename_list[$i], 0, 1 );
      $lchem    = lc($hem);
      $base     = "$nise_tbsdir/$basename_list[$i]";
      $platform = F17;
      $algo     = $algorithm;

      $month_str = sprintf( "%2.2d", $clim_month[$i] );
      $clim_file = "$nise_climdir/${hem}l.ice_climatology.$month_str";

      $landmask   = "$nise_ancillary/${hem}l_lo_land50_coast0km.721x721.bin";
      $weightmask = "$nise_ancillary/${hem}l.cmin_goddard_1992.${hem}11.byte";

      $test_file    = tmp_test_file_name( $tmp_test_dir, $base, $current_ext[$i] );
      $compare_file = $current_compare_list[$i];
      $description  = "\ntest_pmalgos_nasateam $platform ${algo} NISE comparison for basename $base\n" . "using pmalgos at $pmalgos\n";

      $cmd = $cmds[$i];
      if ( $cmd =~ /LANDMASK/ ) {
         $cmd =~ s/LANDMASK/$landmask/;
      }
      if ( $cmd =~ /WEIGHTMASK/ ) {
         $cmd =~ s/WEIGHTMASK/$weightmask/;
      }
      $cmd =~ s/HEM/$hem/;
      $cmd =~ s/PLATFORM/$platform/;
      $cmd =~ s/CLIMFILE/$clim_file/;
      $cmd = "$cmd ${hem}l.gpd ${algo} $base";
      print "Calling: $cmd | $fdistill -u > $test_file\n";
      `$cmd | $fdistill -u > $test_file`;
      check_for_success( $description, $?, $test_file, $compare_file );
   }
}


# Remove your temp directory if you exited fine.
rmdir($tmp_test_dir);

1;


# Subroutines below
# -----------------
sub tmp_test_file_name($$$) {
	my $tmp_dir           = shift;
	my $compare_file_base = shift;
	my $extension         = shift;
	my $base_id           = basename($compare_file_base);
	return catfile( $tmp_test_dir, $base_id . $extension );
}
