#!/usr/local/bin/perl -w
#
# This perl script will produce Goddard time series data for regression testing.
#
# 2006-06-30 M. J. Brodzik 303-492-8263 brodzik@nsidc.org
# National Snow & Ice Data Center, Boulder, CO
# Copyright (C) 2006 University of Colorado
#
#$Id: test_seaice_data.pl,v 1.2.6.1 2008-06-10 20:36:35 brodzik Exp $
#
#-----------------------------------------------------------------------
#Usage: test_seaice_data.pl (no parameters)
#
use Env;

# force autoflush after every write or print
$| = 1;

$topdir = "$HOME";
$map_dir = ${topdir} . "/lib/maps";
$ancillary_dir = ${topdir} . "/src/pmalgos_test/ancillary";
$pmalgos = "${topdir}/src/branches/pmalgos/pmalgos -r -u 0 ";
$fdistill = "${topdir}/bin/fdistill -u ";

$tbsdir = "${topdir}/src/pmalgos_test/tbs/goddard";
                  
@hems      = ( N, S );      
@dates     = ( '880901', '900301',
               '940901', '930301',
               '20000901', '20030301',
           );
%platforms = (
    '880901' => F08,
    '900301' => F08,
    '940901' => F11,
    '930301' => F11,
    '20000901' => F13,
    '20030301' => F13,
    );

$fill_value = 251;

$ENV{"PATHMPP"} = $ENV{"PATHMPP"} . ":${map_dir}";

# Test Goddard Time Series for each March/Septbasename...
$algo = "GODDARD";
foreach $date (@dates) {
    foreach $hem ( @hems ) {

        $lchem = lc($hem);

        $base = "${tbsdir}/${hem}3B.$date";
        print "Next basename  = $base\n";

        $test_file = $base . ".${algo}.tot";
        print "Outfile       = $test_file\n";
        $cmd = "$pmalgos -f $fill_value -l ${ancillary_dir}/t_mask.${lchem}3b "
            . "-q ${hem} -p $platforms{$date} ";
        $cmd = "$cmd ${hem}3B.gpd ${algo}_SEAICE $base";
#        print "Next command is: $cmd\n";
        `$cmd | $fdistill > $test_file`;
        if (0 != $?) {
            die "ERROR: return was not EXIT_SUCCESS, prog_ret = " . 
                ($? >> 8) . ", wait_ret = " . ($? & 255) . "\n";
        } else {
            print "SUCCESS: returned EXIT_SUCCESS\n";
            print "------------------------------------------------\n\n";
        }
    }
}

1;




