################################################################################
#@***h* Sea_Ice_CDR/cdr_configuration_test.py
#
# NAME
#  cdr_configuration_test.py
#
# LOCATION
#  integration_tests/cdr_configuration_test.py
#
# PURPOSE
#  integration tests for testing the cdr configuration code
#
# DESCRIPTION
#  integration tests for testing the cdr configuration code
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2011 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Aug 2011
#
# EXTERNALS
#    os
#    unittest
#    ConfigParser
#    cdr_configuration.configuration_dictionary
#    cdr_configuration.UserDefaults
#    cdr_lib
#
# SUBROUTINES
#   class CdrConfigurationTest
#     test_defaults_as_dict
#     test_nsidc_default_config
#     test_program_defaults_override_user
#     test_simple_config_substitution
#     test_simple_configparser
#
# INPUTS
#  n/a
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################

import os
import unittest
import ConfigParser


from cdr_configuration import configuration_dictionary
from cdr_configuration import UserDefaults

import cdr_lib as cl


class CdrConfigurationTest(unittest.TestCase):

    def test_simple_configparser(self):
        cp = ConfigParser.ConfigParser()
        test_cfg = os.path.join(cl.cdr_root_dir(),
                                "test_data", "configtest", "configtest.cfg")
        cp.read(test_cfg)
        nt_base = cp.get("DEFAULT", "nt_final_base")
        self.assertEqual(nt_base, "this_is_the_project_final_base")

    def test_simple_config_substitution(self):
        root = cl.cdr_root_dir()
        cp = ConfigParser.SafeConfigParser({'CDR_root': root})
        test_cfg = os.path.join(root, "test_data", "configtest", "configtest.cfg")
        cp.read(test_cfg)
        sub_st = cp.get("SubTest", "sub_test")
        self.assertEqual(sub_st, os.path.join(root, "bin"))
        sub_st = cp.get("DEFAULT", "interp_here")
        self.assertEqual(sub_st, os.path.join(root, "maybe"))

    def test_defaults_as_dict(self):
        """ Test that we can convert the parser into a dictionary """

        root = cl.cdr_root_dir()
        cp = ConfigParser.SafeConfigParser({'CDR_root': root})
        test_cfg = os.path.join(root, "test_data", "configtest", "configtest2.cfg")
        cp.read(test_cfg)
        cp_as_dictonary = UserDefaults().convert_config_to_dict(cp)
        defs = {"cdr_root": root,
                "interp_here": os.path.join(root, "maybe"),
                "nt_final_base": "this_is_the_project_final_base",
                "sub_test": os.path.join(root, "bin")}
        self.assertEqual(cp_as_dictonary, defs)

    def test_nsidc_default_config(self):
        """ Test that what I expect from nsidc defaults are usable as designed """

        root = cl.cdr_root_dir()
        cp = ConfigParser.SafeConfigParser({'CDR_root': root})
        test_cfg = os.path.join(root, "test_data", "configtest", "configtest2.cfg")
        cp.read(test_cfg)
        myConf = UserDefaults().convert_config_to_dict(cp)
        self.assertEqual(myConf["nt_final_base"], "this_is_the_project_final_base")

    def test_program_defaults_override_user(self):
        """ If the user has set a directory in the config file, that we also
        set in our configuration, use the configuration setting.  This seems
        incorrect, but I wanted to test it out. """

        # Program config
        config = configuration_dictionary('n', '20070315')

        # without calling create_configuration_dictionary so we pick up what's
        # directly in the config.cfg file.
        root = cl.cdr_root_dir()
        myDict = {}
        cfg = os.path.join(root, "test_data", "test_cdr.config")
        myDict = UserDefaults().load_user_defaults(cfg, myDict)
        self.assertEqual(myDict["output"], config["output"])
        self.assertEqual(myDict["work_base_dir"], config["work_base_dir"])


if __name__ == '__main__':
    unittest.main()
