################################################################################
#@***h* Sea_Ice_CDR/common_metadata_tests.py
#
# NAME
#  common_metadata_tests.py
#
# LOCATION
#  integration_tests/common_metadata_tests.py
#
# PURPOSE
#  Integration tests that are common to both monthly and daily files.
#
# DESCRIPTION
#  simple integration tests
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2013 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Feb 2013
#
# EXTERNALS
#  datetime
#  getpass
#
# SUBROUTINES
#
# class CommonMetadataTests
#     test_naming_authority
#     test_software_version_id
#     test_metadata_link
#     test_history_attribute
#
# INPUTS
#  n/a
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################
import datetime as dt
import getpass


class CommonMetadataTests(object):

    def test_naming_authority(self):
        expected = "nsidc.org"
        actual = getattr(self.ncobj, 'naming_authority')
        self.assertEqual(expected, actual)

    def test_software_version_id(self):
        expected = "newice.colorado.edu/SubversionRepositories/programmers/seaice_cdr"
        actual = getattr(self.ncobj, 'software_version_id')
        self.assertTrue(expected in actual,
                        "{0} does not contain {1}".format(actual, expected))

    def test_metadata_link(self):
        expected = "http://nsidc.org/api/metadata?id=g02202"
        actual = getattr(self.ncobj, 'Metadata_Link')
        self.assertEqual(expected, actual)

    def test_history_attribute(self):
        expects = ["newice.colorado.edu/SubversionRepositories/programmers/seaice_cdr",
                   getpass.getuser(),
                   dt.datetime.utcnow().strftime("%Y-%m-%d")]
        actual = getattr(self.ncobj, 'history')
        for expected in expects:
            self.assertTrue(expected in actual,
                            "{0} does not contain {1}".format(actual, expected))
