################################################################################
#@***h* Sea_Ice_CDR/integration_testsuite_for_cdr.py
#
# NAME
#  integration_testsuite_for_cdr.py
#
# LOCATION
#  integration_tests/integration_testsuite_for_cdr.py
#
# PURPOSE
#  Combine all integration tests into a single test suite
#
# DESCRIPTION
#  All integration tests for the CDR
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2011 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Aug 2011
#
# EXTERNALS
#   unittest2
#   cdr_stddev_test.CDRStddevTests
#   cdr_configuration_test.CdrConfigurationTest
#   cdr_daily_seaice_test.SequenceFunctionsTest
#   cdr_monthly_seaice_test.MonthlyCdrTest
#   daily_metadata_tests.SouthernDailyMetadataTests
#   daily_metadata_tests.NorthernDailyMetadataTests
#   daily_metadata_tests.MissingVarsDailyMetadataTests
#   monthly_metadata_tests.SouthernMetadataTests
#   monthly_metadata_tests.NorthernMetadataTests
#   monthly_metadata_tests.MissingVarsMonthlyMetadataTests
#
# SUBROUTINES
#   suite
#   load_test_case
#
# INPUTS
#  n/a
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################

import unittest2 as unittest
from cdr_stddev_test import CDRStddevTests
from cdr_configuration_test import CdrConfigurationTest
from cdr_daily_seaice_test import SequenceFunctionsTest
from cdr_monthly_seaice_test import MonthlyCdrTest
from daily_metadata_tests import SouthernDailyMetadataTests
from daily_metadata_tests import NorthernDailyMetadataTests
from daily_metadata_tests import MissingVarsDailyMetadataTests
from monthly_metadata_tests import SouthernMetadataTests
from monthly_metadata_tests import NorthernMetadataTests
from monthly_metadata_tests import MissingVarsMonthlyMetadataTests


def suite():
    """ return the test suite for this project.  This just creates TestSuites
    from each of the TestCases and adds them to one large test that is run.
    """
    cases = []
    cases.append(load_test_case(CdrConfigurationTest))
    cases.append(load_test_case(CDRStddevTests))
    cases.append(load_test_case(SequenceFunctionsTest))
    cases.append(load_test_case(MonthlyCdrTest))
    cases.append(load_test_case(SouthernDailyMetadataTests))
    cases.append(load_test_case(NorthernDailyMetadataTests))
    cases.append(load_test_case(MissingVarsDailyMetadataTests))
    cases.append(load_test_case(SouthernMetadataTests))
    cases.append(load_test_case(NorthernMetadataTests))
    cases.append(load_test_case(MissingVarsMonthlyMetadataTests))
    return unittest.TestSuite(cases)


def load_test_case(test_case):
    return unittest.TestLoader().loadTestsFromTestCase(test_case)


if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    test_suite = suite()
    runner.run(test_suite)
