#########################################################################################
#@***h* Sea_Ice_CDR/cdr_daily_seaice_year_doy.py
#
# NAME
#  cdr_daily_seaice_year_doy.py
#
# LOCATION
#  src/cdr_daily_seaice_year_doy.py
#
# PURPOSE
#  Run processing for creating the Climate Data Record of Average Daily Sea
#  Ice Concentration using the year and day of year as input.
#
# DESCRIPTION
#  This code is a simple wrapper to the main cdr_daily_seaice code that will
#  generate both hemisphere's sea ice data based on the input Year and Day of
#  Year
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2013 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Feb 2013
#
# EXTERNALS
#    argparse
#    datetime
#    cdr_daily_seaice
#
# SUBROUTINES
#   parse_arguments
#
#
# INPUTS
#  n/a
#
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################
import argparse as ap
import datetime as dt

import cdr_daily_seaice as cdr_daily_seaice


def parse_arguments():
    parser = ap.ArgumentParser(
        description="Wrapper to generate cdr daily sea ice files for "
        "North and South hemisphere given a year and a day of year."
    )
    parser.add_argument('yyyy', help='four digit year to process.')
    parser.add_argument('day_of_year',
                        help='day of year to process. '
                        'Days > last day of year are ignored.')
    parser.add_argument('version', help='dataset version number to generate')
    parser.add_argument('revision', help='dataset revision number to generate')
    return parser.parse_args()


if __name__ == '__main__':
    args = parse_arguments()
    doy_offset = int(args.day_of_year) - 1
    date = dt.date(int(args.yyyy), 1, 1) + dt.timedelta(doy_offset)
    if (args.yyyy == date.strftime("%Y")):
        cdr_daily_seaice.main(['_', 'both', date.strftime("%Y%m%d"),
                               args.version, args.revision])
