################################################################################
#@***h* Sea_Ice_CDR/cdr_postprocess.py
#
# NAME
#  cdr_postprocess.py
#
# LOCATION
#  src/cdr_postprocess.py
#
# PURPOSE
#  Fill global attribute 'comment'
#
# DESCRIPTION
#  This module processes all daily CDR files and updates the global attribute
#  'comment' with information about missing varaibles in the files.
#
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2013 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Apr 2013
#
# EXTERNALS
#   argparse
#   cdr_configuration
#   cdr_lib
#
# SUBROUTINES
#   postprocess
#   update_global_missing_attribute
#   parse_arguments
#
# INPUTS
#  1. Daily CDR files for the given year
#
# OUTPUTS
#  Updated daily CDR files
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################
import argparse as ap
import cdr_configuration as cfg
import cdr_lib as cl


def postprocess(year, version, revision):
    preset = {
        "cdr_version": cl.create_product_version_string(version, revision)
    }

    for hemi in ['n', 's']:
        for doy in range(1, cl.days_in_year(year) + 1):
            conf = cfg.configuration_dictionary(hemi, cl.yyyymmdd(year, doy), preset)
            try:
                with cl.cdr_dataset(conf) as cdr:
                    update_global_missing_attribute(cdr)
            except RuntimeError:
                print "Couldn't find input file for {0} {1} {2}".format(hemi, year, doy)
                continue


def update_global_missing_attribute(cdr):
    cdr.setncattr('comment', cl.missing_daily_comment(cdr))


def parse_arguments():
    parser = ap.ArgumentParser(
        description="Update global attribute 'comment' based on current contents of file"
    )
    parser.add_argument('year', type=int, help='Year of daily CDR files to postprocess')
    parser.add_argument('version', help='G02202 dataset version to postprocess')
    parser.add_argument('revision', help='G02202 dataset revision to postprocess')
    return parser.parse_args()

if __name__ == "__main__":
    args = parse_arguments()
    postprocess(args.year, args.version, args.revision)
