################################################################################
#@***h* Sea_Ice_CDR/generate_all_monthly_cdr.py
#
# NAME
#  generate_all_monthly_cdr.py
#
# LOCATION
#  src/generate_all_monthly_cdr.py
#
# PURPOSE
#  Generate all CDR Monthly Data
#
# DESCRIPTION
#  Determine the available CDR Daily data and compute the Monthly CDR data.
#
# AUTHOR
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2012 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Apr 2012
#
# EXTERNALS
#  cdr_configuration
#  cdr_lib
#  cdr_monthly_seaice
#
#
# SUBROUTINES
#   generate_all_monthly_cdr
#
#
# INPUTS
#  n/a
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################

import cdr_configuration as config
import cdr_lib as cl
import cdr_monthly_seaice as cms


def generate_all_monthly_cdr():
    """
    compute all CDR monthly data
    """

    # Create conf for known valid date => only used for determining dailyCDR location.
    conf = config.configuration_dictionary('north', '20000101')
    valid_year_range = cl.get_cdr_valid_year_range(conf)
    valid_years = range(valid_year_range['first_year'], valid_year_range['last_year']+1)
    months = range(1, 13)
    first_day = 1
    dates = list()
    for year in valid_years:
        for month in months:
            dates.append("%4.4d%2.2d%2.2d" % (year, month, first_day))

    dates.sort()
    hemispheres = ['north', 'south']

    for hemi in hemispheres:
        for date in dates:

            try:
                conf = config.configuration_dictionary(hemi, date)
                cms.CDRMonthlySeaice(conf).process_monthly_cdr()
            except IOError as e:
                print("\n Caught exception:\n %s\n" % e)
                print("\n Invalid Date, skipping processing %s" % date)


if __name__ == '__main__':

    generate_all_monthly_cdr()
