#!/bin/bash
#------------------------------------------------------------------------
# make_empty.bash
#     create empty NetCDF4 file for NSIDC Sea-Ice CDR
#
# April-2011  Sean Mallory, sean.mallory@nsidc.org, 303.492.7638
# National Snow & Ice Data Center, University of Colorado, Boulder
#
# Copyright 2011 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
#------------------------------------------------------------------------

# Set defaults
export NC_TYPE="netCDF-4 classic model"

# ${NC_BIN} will wither need to go away or get smart for public release.
if [ ! -e $NC_BIN ] ; then 
    echo ${NC_BIN} not found! Exiting.
    # POSIX failures are non-zero return values
    exit 1
fi


# Check for command line overrides
# CDL file
if [[ $# > 0 ]] ; then
    CDL_NAME=$1
    shift
fi
# NC file
if [[ $# > 0 ]] ; then
    export NC_NAME=$1
    shift
else 
    export NC_NAME=${CDL_NAME%.cdl}.nc
fi
# end of checking for command line overrides

# Sane?
if [ -e $CDL_NAME ] ; then 
    echo Using ${CDL_NAME} to create ${NC_NAME}.
else 
    echo ${CDL_NAME} not found! Exiting.
    # POSIX failures are non-zero return values
    exit 1
fi

if [ -e $NC_NAME ] ; then 
    echo ${NC_NAME} already exists! Exiting instead of overwriting.
    # POSIX failures are non-zero return values
    exit 1
fi
# OK, Sane!

${NC_BIN}/ncgen -k "${NC_TYPE}" -bo ${NC_NAME} ${CDL_NAME}
NCRESULT=`${NC_BIN}/ncdump -k ${NC_NAME}`

# Quick test of the output
if [[ $NCRESULT != $NC_TYPE ]] ; then
	echo Problem creating empty NetCDF file ${NC_NAME}.
else
	echo Empty $NC_TYPE NetCDF file \"${NC_NAME}\" generated.
fi

#
# A return code here makes this useful in a make file.
#
