################################################################################
#@***h* Sea_Ice_CDR/noaa_util.py
#
# NAME
#  noaa_util.py
#
# LOCATION
#  src/noaa_util.py
#
# PURPOSE
#  Utility functions to dump a NOAA-compliant header SUBROUTINES section.
#
# DESCRIPTION
#  This utility can be used to create the SUBROUTINES comment section
#  for the NOAA header by calling it with the name of a module (module
#  must by on the PYTHONPATH), e.g.:
#
#    python src/noaa_util.py cdr_lib
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2011 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  April 2013
#
# EXTERNALS
#   inspect
#   sys
#
# SUBROUTINES
#   classes
#   function_names
#   main
#   method_names
#
# INPUTS
#  n/a
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################
import inspect
import sys


def function_names(module_name):
    the_module = __import__(module_name)
    return [name for name, _ in inspect.getmembers(the_module, inspect.isfunction)]


def method_names(the_module):
    return [name for name, _ in inspect.getmembers(the_module, inspect.ismethod)]


def classes(module_name):
    the_module = __import__(module_name)
    return [[name, method_names(klass)]
            for name, klass
            in inspect.getmembers(the_module, inspect.isclass)]


def main(module_name):
    print "# SUBROUTINES"
    for name in function_names(module_name):
        print "#   %s" % name

    for class_info in classes(module_name):
        print "#   class %s" % class_info[0]
        for method_name in class_info[1]:
            print "#     %s" % method_name


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print "Usage: python noaa_util.py module_name"
    main(sys.argv[1])
