/*
 *@***h* Sea_Ice_CDR/sea_ice.h
 *
 * NAME
 *  sea_ice.h
 *
 * LOCATION
 *  ../src/sea_ice.h
 * 
 * PURPOSE
 *  To provide a header file for decoding Goddard final sea-ice product headers.
 *
 * DESCRIPTION
 *  Describes the in-memory structure of a Goddard final sea-ice product header.
 *      
 * AUTHOR
 *  Sean Mallory, sean.mallory@nsidc.org
 *  National Snow & Ice Data Center, University of Colorado, Boulder
 *
 * CREATION DATE
 *  April 2011
 * 
 * COPYRIGHT
 *   Copyright 2011 Regents of the University of Colorado. The Regents of the
 *   University of Colorado grants, and you accept, a personal, nonexclusive,
 *   nontransferable license to use this software, at no charge, in accordance
 *   with the terms herein, solely for internal research and development,
 *   which is limited to testing, measuring, assessing, evaluating the
 *   software and/or data that you generate using this software, and not for
 *   any commercial purpose. You may not develop derivative works based on
 *   this software, or redistribute this software.
 *
 * EXTERNALS
 *   n/a
 *
 * SUBROUTINES
 *   n/a
 * 
 * INPUTS
 *   n/a
 *
 * OUTPUTS
 *   n/a
 *      
 * MODIFICATION HISTORY
 *   n/a
 *
 *@*****
 */

typedef struct {
  /* Documentation of fields from 
     http://nsidc.org/data/docs/daac/nsidc0051_gsfc_seaice.gd.html#format */
  char missing_data_value[6];      /* 1-6     Missing data integer value */
  char column_count[6];            /* 7-12    Number of columns in polar
				               stereographic grid */
  char row_count[6];               /* 13-18   Number of rows in polar
				               stereographic grid */
  char unused_1[6];                /* 19-24   Unused/internal */
  char enclosed_latitude[6];       /* 25-30   Latitude enclosed by polar
				               stereographic grid */
  char grid_rotation[6];           /* 31-36   Greenwich orientation of polar
				               stereographic grid */
  char unused_2[6];                /* 37-42   Unused/internal */
  char J_pole_coord[6];            /* 43-48   J-coordinate of the grid
				               intersection at the pole */
  char I_pole_coord[6];            /* 49-54   I-coordinate of the grid
				               intersection at the pole */
  char instrument_abbreviation[6]; /* 55-60   Five-character instrument
				               descriptor:
                                               SMMR, SSM/I, SSMIS) */
  char data_descriptors[6];        /* 61-66   Two descriptors of two characters
				               each that describe the data;
                                               for example, 
					       07 cn = Nimbus-7 SMMR ice
					               concentration */
  char grid_start_day_of_year[6];  /* 67-72   Starting day of year of grid
				              data */
  char grid_start_hour[6];         /* 73-78   Starting hour of grid data (if
				              available) -9999 fill value */
  char grid_start_minute[6];       /* 79-84   Starting minute of grid data (if
                                                available) -9999 fill value */
  char grid_end_day_of_year[6];    /* 85-90   Ending day of year of grid data */
  char grid_end_hour[6];           /* 91-96   Ending hour of grid data (if
				              available) -9999 fill value */
  char grid_end_minute[6];         /* 97-102  Ending minute of grid data (if
                                               available) -9999 fill value */
  char grid_data_year[6];          /* 103-108 Year of grid data */
  char grid_data_day_of_year[6];   /* 109-114 Day of year of grid data */
  char channel_descriptor[6];      /* 115-120 Three-digit channel descriptor:
                                               000 for ice concentrations) */
  char scaling_factor[6];          /* 121-126 Integer scaling factor */
  char file_name[24];              /* 127-150 24-character file name */
  char image_title[80];            /* 151-230 80-character image title */
  char data_information[70];       /* 231-300 70-character data information
                                               (creation date, data source,
					        etc.) */
} nasateam_header;
