################################################################################
#@***h* Sea_Ice_CDR/test_cdr_daily_seaice.py
#
# NAME
#  test_cdr_daily_seaice.py
#
# LOCATION
#  tests/test_cdr_daily_seaice.py
#
# PURPOSE
#  unit tests for cdr_daily_seaice.py
#
# DESCRIPTION
#  simple unit tests
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2011 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Aug 2011
#
# EXTERNALS
#   __future__.division
#   __future__.print_function
#   __future__.absolute_import
#   numpy
#   unittest2
#   cdr_daily_seaice
#
# SUBROUTINES
#   class TestDailyFilterFactories
#        test_bootstrap_filter_factory
#        test_nasateam_filter_factory
#
# INPUTS
#  n/a
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################

# import a few __future__ features to keep possible upgrades cleaner
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import


import numpy
import unittest2 as unittest


import cdr_daily_seaice as cdr


class TestDailyFilterFactories(unittest.TestCase):

    def test_bootstrap_filter_factory(self):
        """
        Test that a raw array passed into bootstrap_value_filter returns expected results.
        """

        raw = numpy.asarray([0, 1, 100, 200, 300, 305, 315, 1100, 1200, ], dtype='uint16')
        expected = numpy.asarray([0,   0,  10,  20,  30,  31,  32, 255, 254],
                                 dtype='uint16')
        cdr.bootstrap_value_filter(raw)
        numpy.testing.assert_array_equal(expected, raw)

    def test_nasateam_filter_factory(self):
        """
        Test that a raw array passed into bootstrap_value_filter returns expected results.
        """

        conf = {}
        conf["smallest_flag_value"] = 251
        cdr.conf = conf
        raw = numpy.asarray([0,   1,  37, 100, 200, 251, 254], dtype='uint8')
        expected = numpy.asarray([0,   0,  15,  40,  80, 251, 254], dtype='uint8')
        cdr.nasateam_value_filter(raw)
        numpy.testing.assert_array_equal(expected, raw)

if __name__ == '__main__':
    unittest.main()
