################################################################################
#@***h* Sea_Ice_CDR/unit_test_suite_for_cdr.py
#
# NAME
#  unit_test_suite_for_cdr.py
#
# LOCATION
#  tests/unit_test_suite_for_cdr.py
#
# PURPOSE
#  Combine all unittests into a single test suite
#
# DESCRIPTION
#  Combine all unittests into a single test suite
#
# AUTHOR
#  nsidc@nsidc.org
#  National Snow & Ice Data Center, University of Colorado, Boulder
#
# COPYRIGHT
#  Copyright 2011 Regents of the University of Colorado. The Regents of the
#  University of Colorado grants, and you accept, a personal, nonexclusive,
#  nontransferable license to use this software, at no charge, in accordance
#  with the terms herein, solely for internal research and development, which
#  is limited to testing, measuring, assessing, evaluating the software and/or
#  data that you generate using this software, and not for any commercial
#  purpose. You may not develop derivative works based on this software, or
#  redistribute this software.
#
# CREATION DATE
#  Aug 2011
#
# EXTERNALS
#   unittest2
#   melting_tests.MeltingTests
#   test_cdr_stddev.CDRStddevTests
#   test_cdr_configuration.TestCdrConfiguration
#   test_cdr_lib.TestCDRLib
#   test_cdr_daily_seaice.TestDailyFilterFactories
#   test_cdr_monthly_seaice.TestMonthlyCdr
#   test_noaa_util.NOAAUtilTests
#
# SUBROUTINES
#   suite
#
#
# INPUTS
#  n/a
#
# OUTPUTS
#  n/a
#
# MODIFICATION HISTORY
#
#
#@*****
################################################################################

import unittest2 as unittest
from melting_tests import MeltingTests
from test_cdr_stddev import CDRStddevTests
from test_cdr_configuration import TestCdrConfiguration
from test_cdr_lib import TestCDRLib
from test_cdr_daily_seaice import TestDailyFilterFactories
from test_cdr_monthly_seaice import TestMonthlyCdr
from test_noaa_util import NOAAUtilTests


def suite():
    """ return the test suite for this project.  This just creates TestSuites
    from each of the TestCases and adds them to one large test that is run.
    """
    cases = []
    cases.append(unittest.TestLoader().loadTestsFromTestCase(MeltingTests))
    cases.append(unittest.TestLoader().loadTestsFromTestCase(CDRStddevTests))
    cases.append(unittest.TestLoader().loadTestsFromTestCase(TestCdrConfiguration))
    cases.append(unittest.TestLoader().loadTestsFromTestCase(TestCDRLib))
    cases.append(unittest.TestLoader().loadTestsFromTestCase(TestDailyFilterFactories))
    cases.append(unittest.TestLoader().loadTestsFromTestCase(TestMonthlyCdr))
    cases.append(unittest.TestLoader().loadTestsFromTestCase(NOAAUtilTests))

    return unittest.TestSuite(cases)


if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    test_suite = suite()
    runner.run(test_suite)
